@@ -14,7 +14,7 @@ import Utility.FreeDesktop
 #ifdef darwin_HOST_OS
 import Utility.OSX
 import Utility.Path
-import Utility.Directory
+import Utility.SystemDirectory
 import Utility.FileSystemEncoding
 #endif
 
