@@ -1,11 +1,11 @@
 {- git-annex assistant commit thread
  -
- - Copyright 2012-2021 Joey Hess <id@joeyh.name>
+ - Copyright 2012-2023 Joey Hess <id@joeyh.name>
  -
  - Licensed under the GNU AGPL version 3 or higher.
  -}
 
-{-# LANGUAGE CPP #-}
+{-# LANGUAGE CPP, OverloadedStrings #-}
 
 module Assistant.Threads.Committer where
 
@@ -38,14 +38,18 @@ import qualified Annex
 import Utility.InodeCache
 import qualified Database.Keys
 import qualified Command.Sync
+import qualified Command.Add
+import Config.GitConfig
 import Utility.Tuple
 import Utility.Metered
+import qualified Utility.RawFilePath as R
 
 import Data.Time.Clock
 import qualified Data.Set as S
 import qualified Data.Map as M
 import Data.Either
 import Control.Concurrent
+import System.PosixCompat.Files (fileID, deviceID, fileMode)
 
 {- This thread makes git commits at appropriate times. -}
 commitThread :: NamedThread
@@ -54,6 +58,7 @@ commitThread = namedThread "Committer" $ do
 	delayadd <- liftAnnex $
 		fmap Seconds . annexDelayAdd <$> Annex.getGitConfig
 	largefilematcher <- liftAnnex largeFilesMatcher
+	annexdotfiles <- liftAnnex $ getGitConfigVal annexDotFiles
 	msg <- liftAnnex Command.Sync.commitMsg
 	lockdowndir <- liftAnnex $ fromRepo gitAnnexTmpWatcherDir
 	liftAnnex $ do
@@ -63,7 +68,7 @@ commitThread = namedThread "Committer" $ do
 			(fromRawFilePath lockdowndir)
 		void $ createAnnexDirectory lockdowndir
 	waitChangeTime $ \(changes, time) -> do
-		readychanges <- handleAdds (fromRawFilePath lockdowndir) havelsof largefilematcher delayadd $
+		readychanges <- handleAdds (fromRawFilePath lockdowndir) havelsof largefilematcher annexdotfiles delayadd $
 			simplifyChanges changes
 		if shouldCommit False time (length readychanges) readychanges
 			then do
@@ -93,7 +98,7 @@ refill cs = do
  - runs an action to commit them. If more changes arrive while this is
  - going on, they're handled intelligently, batching up changes into
  - large commits where possible, doing rename detection, and
- - commiting immediately otherwise. -}
+ - committing immediately otherwise. -}
 waitChangeTime :: (([Change], UTCTime) -> Assistant Int) -> Assistant ()
 waitChangeTime a = waitchanges 0
   where
@@ -259,8 +264,8 @@ commitStaged msg = do
  - Any pending adds that are not ready yet are put back into the ChangeChan,
  - where they will be retried later.
  -}
-handleAdds :: FilePath -> Bool -> GetFileMatcher -> Maybe Seconds -> [Change] -> Assistant [Change]
-handleAdds lockdowndir havelsof largefilematcher delayadd cs = returnWhen (null incomplete) $ do
+handleAdds :: FilePath -> Bool -> GetFileMatcher -> Bool -> Maybe Seconds -> [Change] -> Assistant [Change]
+handleAdds lockdowndir havelsof largefilematcher annexdotfiles delayadd cs = returnWhen (null incomplete) $ do
 	let (pending, inprocess) = partition isPendingAddChange incomplete
 	let lockdownconfig = LockDownConfig
 		{ lockingFile = False
@@ -287,15 +292,22 @@ handleAdds lockdowndir havelsof largefilematcher delayadd cs = returnWhen (null
 		| c = return otherchanges
 		| otherwise = a
 
-	checksmall change =
-		ifM (liftAnnex $ checkFileMatcher largefilematcher (toRawFilePath (changeFile change)))
-			( return (Left change)
-			, return (Right change)
-			)
+	checksmall change
+		| not annexdotfiles && dotfile f =
+			return (Right change)
+		| otherwise =
+			ifM (liftAnnex $ checkFileMatcher largefilematcher f)
+				( return (Left change)
+				, return (Right change)
+				)
+	  where
+		f = toRawFilePath (changeFile change)
 
 	addsmall [] = noop
-	addsmall toadd = liftAnnex $ Annex.Queue.addCommand [] "add"
-		[ Param "--force", Param "--"] (map changeFile toadd)
+	addsmall toadd = liftAnnex $ void $ tryIO $
+		forM (map (toRawFilePath . changeFile) toadd) $ \f ->
+			Command.Add.addFile Command.Add.Small f
+				=<< liftIO (R.getSymbolicLinkStatus f)
 
 	{- Avoid overhead of re-injesting a renamed unlocked file, by
 	 - examining the other Changes to see if a removed file has the
@@ -331,7 +343,7 @@ handleAdds lockdowndir havelsof largefilematcher delayadd cs = returnWhen (null
 	  	ks = keySource ld
 		doadd = sanitycheck ks $ do
 			(mkey, _mcache) <- liftAnnex $ do
-				showStart "add" (keyFilename ks) (SeekInput [])
+				showStartMessage (StartMessage "add" (ActionItemOther (Just (QuotedPath (keyFilename ks)))) (SeekInput []))
 				ingest nullMeterUpdate (Just $ LockedDown lockdownconfig ks) Nothing
 			maybe (failedingest change) (done change $ fromRawFilePath $ keyFilename ks) mkey
 	addannexed' _ _ = return Nothing
@@ -358,7 +370,7 @@ handleAdds lockdowndir havelsof largefilematcher delayadd cs = returnWhen (null
 
 	done change file key = liftAnnex $ do
 		logStatus key InfoPresent
-		mode <- liftIO $ catchMaybeIO $ fileMode <$> getFileStatus file
+		mode <- liftIO $ catchMaybeIO $ fileMode <$> R.getFileStatus (toRawFilePath file)
 		stagePointerFile (toRawFilePath file) mode =<< hashPointerFile key
 		showEndOk
 		return $ Just $ finishedChange change key
@@ -367,8 +379,8 @@ handleAdds lockdowndir havelsof largefilematcher delayadd cs = returnWhen (null
 	 - and is still a hard link to its contentLocation,
 	 - before ingesting it. -}
 	sanitycheck keysource a = do
-		fs <- liftIO $ getSymbolicLinkStatus $ fromRawFilePath $ keyFilename keysource
-		ks <- liftIO $ getSymbolicLinkStatus $ fromRawFilePath $ contentLocation keysource
+		fs <- liftIO $ R.getSymbolicLinkStatus $ keyFilename keysource
+		ks <- liftIO $ R.getSymbolicLinkStatus $ contentLocation keysource
 		if deviceID ks == deviceID fs && fileID ks == fileID fs
 			then a
 			else do
@@ -431,8 +443,8 @@ safeToAdd lockdowndir lockdownconfig havelsof delayadd pending inprocess = do
 
 	canceladd (InProcessAddChange { lockedDown = ld }) = do
 		let ks = keySource ld
-		warning $ fromRawFilePath (keyFilename ks)
-			++ " still has writers, not adding"
+		warning $ QuotedPath (keyFilename ks)
+			<> " still has writers, not adding"
 		-- remove the hard link
 		when (contentLocation ks /= keyFilename ks) $
 			void $ liftIO $ tryIO $ removeFile $ fromRawFilePath $ contentLocation ks
