@@ -319,7 +319,7 @@ testServer sshinput@(SshInput { inputHostname = Just hn }) = do
 		finduuid (k, v)
 			| k == "annex.uuid" = Just $ toUUID v
 			| k == fromConfigKey GCrypt.coreGCryptId =
-				Just $ genUUIDInNameSpace gCryptNameSpace v
+				Just $ genUUIDInNameSpace gCryptNameSpace (encodeBS v)
 			| otherwise = Nothing
 	
 	checkcommand c = "if which " ++ c ++ "; then " ++ report c ++ "; fi"
@@ -642,7 +642,7 @@ enableRsyncNetGCrypt sshinput reponame =
  - used on rsync.net. If successful, runs an action with its SshData.
  -
  - To append the ssh key to rsync.net's authorized_keys, their
- - documentation recommends a dd methodd, where the line is fed
+ - documentation recommends a dd method, where the line is fed
  - in to ssh over stdin.
  -}
 prepRsyncNet :: SshInput -> String -> (SshData -> Handler Html) -> Handler Html
