@@ -115,7 +115,17 @@ batchLines (BatchFormat sep _) = do
   where
 	splitter = case sep of
 		BatchLine -> lines
-		BatchNull -> splitc '\0'
+		BatchNull -> elimemptyend . splitc '\0'
+
+	-- When there is a trailing null on the input, eliminate the empty
+	-- string that splitc generates. Other empty strings elsewhere in
+	-- the list are preserved. This is the same effect as how `lines`
+	-- handles a trailing newline.
+	elimemptyend [] = []
+	elimemptyend (x:[])
+		| null x = []
+		| otherwise = [x]
+	elimemptyend (x:rest) = x : elimemptyend rest
 
 -- When concurrency is enabled at the command line, it is used in batch
 -- mode. But, if it's only set in git config, don't use it, because the
@@ -154,9 +164,8 @@ batchFilesKeys fmt a = do
 	matcher <- getMatcher
 	go $ \si v -> case v of
 		Right f -> 
-			let f' = toRawFilePath f
-			in ifM (matcher $ MatchingFile $ FileInfo f' f' Nothing)
-				( a (si, Right f')
+			ifM (matcher $ MatchingFile $ FileInfo f f Nothing)
+				( a (si, Right f)
 				, return Nothing
 				)
 		Left k -> a (si, Left k)
@@ -167,7 +176,7 @@ batchFilesKeys fmt a = do
 		-- because in non-batch mode, that is done when
 		-- CmdLine.Seek uses git ls-files.
 		BatchFormat _ (BatchKeys False) -> 
-			Right . Right . fromRawFilePath 
+			Right . Right
 				<$$> liftIO . relPathCwdToFile . toRawFilePath
 		BatchFormat _ (BatchKeys True) -> \i ->
 			pure $ case deserializeKey i of
@@ -189,7 +198,7 @@ batchAnnexed fmt seeker keyaction = do
 			Right f -> lookupKeyStaged f >>= \case
 				Nothing -> return Nothing
 				Just k -> checkpresent k $
-					startAction seeker si f k
+					startAction seeker Nothing si f k
 			Left k -> ifM (matcher (MatchingInfo (mkinfo k)))
 				( checkpresent k $
 					keyaction (si, k, mkActionItem k)
