@@ -5,6 +5,8 @@
  - Licensed under the GNU AGPL version 3 or higher.
  -}
 
+{-# LANGUAGE OverloadedStrings #-}
+
 module Command.Add where
 
 import Command
@@ -84,7 +86,10 @@ checkGitIgnoreSwitch = CheckGitIgnore <$>
 		(help "Do not check .gitignore when adding files")
 
 seek :: AddOptions -> CommandSeek
-seek o = startConcurrency commandStages $ do
+seek o = startConcurrency commandStages (seek' o)
+
+seek' :: AddOptions -> CommandSeek
+seek' o = do
 	largematcher <- largeFilesMatcher
 	addunlockedmatcher <- addUnlockedMatcher
 	annexdotfiles <- getGitConfigVal annexDotFiles 
@@ -117,7 +122,7 @@ seek o = startConcurrency commandStages $ do
 			-- are not known to git yet, since this will add
 			-- them. Instead, have workTreeItems warn about other
 			-- problems, like files that don't exist.
-			let ww = WarnUnmatchWorkTreeItems
+			let ww = WarnUnmatchWorkTreeItems "add"
 			l <- workTreeItems ww (addThese o)
 			let go b a = a ww (commandAction . gofile b) l
 			unless (updateOnly o) $
@@ -160,7 +165,10 @@ addFile smallorlarge file s = do
 		then hashBlob =<< liftIO (R.readSymbolicLink file)
 		else if isRegularFile s
 			then hashFile file
-			else giveup $ fromRawFilePath file ++ " is not a regular file"
+			else do
+				qp <- coreQuotePath <$> Annex.getGitConfig
+				giveup $ decodeBS $ quote qp $
+					file <> " is not a regular file"
 	let treetype = if isSymbolicLink s
 		then TreeSymlink
 		else if intersectFileModes ownerExecuteMode (fileMode s) /= 0
@@ -169,7 +177,7 @@ addFile smallorlarge file s = do
 	s' <- liftIO $ catchMaybeIO $ R.getSymbolicLinkStatus file
 	if maybe True (changed s) s'
 		then do
-			warning $ fromRawFilePath file ++ " changed while it was being added"
+			warning $ QuotedPath file <> " changed while it was being added"
 			return False
 		else do
 			case smallorlarge of
