@@ -24,7 +24,7 @@ cmd :: Command
 cmd = dontCheck repoExists $ notBareRepo $
 	noRepo (startNoRepo <$$> optParser) $
 		command "assistant" SectionCommon
-			"automatically sync changes"
+			"daemon to add files and automatically sync changes"
 			paramNothing (seek <$$> optParser)
 
 data AssistantOptions = AssistantOptions
@@ -130,8 +130,11 @@ autoStop = do
 	program <- programPath
 	forM_ dirs $ \d -> do
 		putStrLn $ "git-annex autostop in " ++ d
-		setCurrentDirectory d
-		ifM (boolSystem program [Param "assistant", Param "--stop"])
-			( putStrLn "ok"
-			, putStrLn "failed"
-			)
+		tryIO (setCurrentDirectory d) >>= \case
+			Right () -> ifM (boolSystem program [Param "assistant", Param "--stop"])
+				( putStrLn "ok"
+				, putStrLn "failed"
+				)
+			Left e -> do
+				putStrLn (show e)
+				putStrLn "failed"
