@@ -13,7 +13,7 @@ import Logs.Location
 import Annex.Content
 
 cmd :: Command
-cmd = noCommit $ withGlobalOptions [jsonOptions] $
+cmd = noCommit $ withAnnexOptions [jsonOptions] $
 	command "dropkey" SectionPlumbing
 		"drops annexed content for specified keys"
 		(paramRepeating paramKey)
@@ -27,17 +27,17 @@ data DropKeyOptions = DropKeyOptions
 optParser :: CmdParamsDesc -> Parser DropKeyOptions
 optParser desc = DropKeyOptions
 	<$> cmdParams desc
-	<*> parseBatchOption
+	<*> parseBatchOption False
 
 seek :: DropKeyOptions -> CommandSeek
 seek o = do
-	unlessM (Annex.getState Annex.force) $
+	unlessM (Annex.getRead Annex.force) $
 		giveup "dropkey can cause data loss; use --force if you're sure you want to do this"
-	withKeys (commandAction . start) (toDrop o)
 	case batchOption o of
-		Batch fmt -> batchInput fmt (pure . parsekey) $
-			batchCommandAction . start
-		NoBatch -> noop
+		NoBatch -> withKeys (commandAction . start) (toDrop o)
+		Batch fmt -> batchOnly Nothing (toDrop o) $
+			batchInput fmt (pure . parsekey) $
+				batchCommandAction . start
   where
 	parsekey = maybe (Left "bad key") Right . deserializeKey
 
