@@ -1,6 +1,6 @@
 {- git-annex command
  -
- - Copyright 2013-2020 Joey Hess <id@joeyh.name>
+ - Copyright 2013-2023 Joey Hess <id@joeyh.name>
  -
  - Licensed under the GNU AGPL version 3 or higher.
  -}
@@ -30,10 +30,11 @@ import Git.Config
 import qualified Data.Map as M
 
 cmd :: Command
-cmd = command "enableremote" SectionSetup
-	"enables git-annex to use a remote"
-	(paramPair paramName $ paramOptional $ paramRepeating paramParamValue)
-	(withParams seek)
+cmd = withAnnexOptions [jsonOptions] $
+	command "enableremote" SectionSetup
+		"enables git-annex to use a remote"
+		(paramPair paramName $ paramOptional $ paramRepeating paramParamValue)
+		(withParams seek)
 
 seek :: CmdParams -> CommandSeek
 seek = withWords (commandAction . start)
@@ -43,70 +44,78 @@ start [] = unknownNameError "Specify the remote to enable."
 start (name:rest) = go =<< filter matchingname <$> Annex.getGitRemotes
   where
 	matchingname r = Git.remoteName r == Just name
-	go [] = 
-		let use = startSpecialRemote name (Logs.Remote.keyValToConfig Proposed rest)
-		in SpecialRemote.findExisting' name >>= \case
-			-- enable dead remote only when there is no
-			-- other remote with the same name
-			([], l) -> use l
-			(l, _) -> use l
-	go (r:_) = do
-		-- This could be either a normal git remote or a special
-		-- remote that has an url (eg gcrypt).
-		rs <- Remote.remoteList
-		case filter (\rmt -> Remote.name rmt == name) rs of
-			(rmt:_) | Remote.remotetype rmt == Remote.Git.remote ->
-				startNormalRemote name rest r
-			_  -> go []
+	go [] = deadLast name $ 
+		startSpecialRemote name (Logs.Remote.keyValToConfig Proposed rest)
+	go (r:_)
+		| not (null rest) = go []
+		| otherwise = do
+			-- This could be either a normal git remote or a special
+			-- remote that has an url (eg gcrypt).
+			rs <- Remote.remoteList
+			case filter (\rmt -> Remote.name rmt == name) rs of
+				(rmt:_) | Remote.remotetype rmt == Remote.Git.remote ->
+					startNormalRemote name r
+				_  -> go []
 
--- Normal git remotes are special-cased; enableremote retries probing
--- the remote uuid.
-startNormalRemote :: Git.RemoteName -> [String] -> Git.Repo -> CommandStart
-startNormalRemote name restparams r
-	| null restparams = starting "enableremote" ai si $ do
-		setRemoteIgnore r False
-		r' <- Remote.Git.configRead False r
-		u <- getRepoUUID r'
-		next $ return $ u /= NoUUID
-	| otherwise = giveup $
-		"That is a normal git remote; passing these parameters does not make sense: " ++ unwords restparams
+-- enableremote of a normal git remote with no added parameters is a special case
+-- that retries probing the remote uuid.
+startNormalRemote :: Git.RemoteName -> Git.Repo -> CommandStart
+startNormalRemote name r = starting "enableremote (normal)" ai si $ do
+	setRemoteIgnore r False
+	r' <- Remote.Git.configRead False r
+	u <- getRepoUUID r'
+	next $ return $ u /= NoUUID
   where
-	ai = ActionItemOther (Just name)
+	ai = ActionItemOther (Just (UnquotedString name))
 	si = SeekInput [name]
 
 startSpecialRemote :: Git.RemoteName -> Remote.RemoteConfig -> [(UUID, Remote.RemoteConfig, Maybe (SpecialRemote.ConfigFrom UUID))] -> CommandStart
-startSpecialRemote name config [] = do
+startSpecialRemote = startSpecialRemote' "enableremote" performSpecialRemote
+
+type PerformSpecialRemote = RemoteType -> UUID -> R.RemoteConfig -> R.RemoteConfig -> RemoteGitConfig -> Maybe (SpecialRemote.ConfigFrom UUID) -> CommandPerform
+
+startSpecialRemote' :: String -> PerformSpecialRemote -> Git.RemoteName -> Remote.RemoteConfig -> [(UUID, Remote.RemoteConfig, Maybe (SpecialRemote.ConfigFrom UUID))] -> CommandStart
+startSpecialRemote' cname perform name config [] = do
 	m <- SpecialRemote.specialRemoteMap
 	confm <- Logs.Remote.remoteConfigMap
 	Remote.nameToUUID' name >>= \case
 		([u], _) | u `M.member` m ->
-			startSpecialRemote name config $
+			startSpecialRemote' cname perform name config $
 				[(u, fromMaybe M.empty (M.lookup u confm), Nothing)]
 		(_, msg) -> unknownNameError msg
-startSpecialRemote name config ((u, c, mcu):[]) =
-	starting "enableremote" ai si $ do
+startSpecialRemote' cname perform name config ((u, c, mcu):[]) =
+	starting cname ai si $ do
 		let fullconfig = config `M.union` c	
 		t <- either giveup return (SpecialRemote.findType fullconfig)
 		gc <- maybe (liftIO dummyRemoteGitConfig) 
 			(return . Remote.gitconfig)
 			=<< Remote.byUUID u
-		performSpecialRemote t u c fullconfig gc mcu
+		perform t u c fullconfig gc mcu
   where
-	ai = ActionItemOther (Just name)
+	ai = ActionItemOther (Just (UnquotedString name))
 	si = SeekInput [name]
-startSpecialRemote _ _ _ =
-	giveup "Multiple remotes have that name. Either use git-annex renameremote to rename them, or specify the uuid of the remote to enable."
+startSpecialRemote' _ _ _ _ _ =
+	giveup "Multiple remotes have that name. Either use git-annex renameremote to rename them, or specify the uuid of the remote."
 
-performSpecialRemote :: RemoteType -> UUID -> R.RemoteConfig -> R.RemoteConfig -> RemoteGitConfig -> Maybe (SpecialRemote.ConfigFrom UUID) -> CommandPerform
+performSpecialRemote :: PerformSpecialRemote
 performSpecialRemote t u oldc c gc mcu = do
+	-- Avoid enabling a special remote if there is another remote
+	-- with the same name.
+	case SpecialRemote.lookupName c of
+		Nothing -> noop
+		Just name -> do
+			rs <- Remote.remoteList
+			case filter (\rmt -> Remote.name rmt == name) rs of
+				(rmt:_) | Remote.uuid rmt /= u ->
+					giveup $ "Not overwriting currently configured git remote named \"" ++ name ++ "\""
+				_ -> noop
 	(c', u') <- R.setup t (R.Enable oldc) (Just u) Nothing c gc
 	next $ cleanupSpecialRemote t u' c' mcu
 
 cleanupSpecialRemote :: RemoteType -> UUID -> R.RemoteConfig -> Maybe (SpecialRemote.ConfigFrom UUID) -> CommandCleanup
 cleanupSpecialRemote t u c mcu = do
 	case mcu of
-		Nothing -> 
-			Logs.Remote.configSet u c
+		Nothing -> Logs.Remote.configSet u c
 		Just (SpecialRemote.ConfigFrom cu) -> do
 			setConfig (remoteAnnexConfig c "config-uuid") (fromUUID cu)
 			Logs.Remote.configSet cu c
@@ -140,3 +149,11 @@ unknownNameError prefix = do
 		, liftIO . getDynamicConfig . remoteAnnexIgnore
 			=<< Annex.getRemoteGitConfig r
 		]
+
+-- Use dead remote only when there is no other remote
+-- with the same name
+deadLast :: Git.RemoteName -> ([(UUID, Remote.RemoteConfig, Maybe (SpecialRemote.ConfigFrom UUID))] -> Annex a) -> Annex a
+deadLast name use =
+	SpecialRemote.findExisting' name >>= \case
+		([], l) -> use l
+		(l, _) -> use l
