@@ -12,7 +12,7 @@ import qualified Command.Find as Find
 import qualified Git
 
 cmd :: Command
-cmd = withGlobalOptions [annexedMatchingOptions] $ Find.mkCommand $ 
+cmd = withAnnexOptions [annexedMatchingOptions] $ Find.mkCommand $ 
 	command "findref" SectionPlumbing
 		"lists files in a git ref (deprecated)"
 		paramRef (seek <$$> Find.optParser)
@@ -22,6 +22,6 @@ seek o = Find.seek o'
   where
 	o' = o 
 		{ Find.keyOptions = Just $ WantBranchKeys $
-			map (Git.Ref . encodeBS') (Find.findThese o)
+			map (Git.Ref . encodeBS) (Find.findThese o)
 		, Find.findThese = []
 		}
