@@ -1,6 +1,6 @@
 {- git-annex command
  -
- - Copyright 2011-2022 Joey Hess <id@joeyh.name>
+ - Copyright 2011-2023 Joey Hess <id@joeyh.name>
  -
  - Licensed under the GNU AGPL version 3 or higher.
  -}
@@ -13,6 +13,7 @@ import "mtl" Control.Monad.State.Strict
 import qualified Data.Map.Strict as M
 import qualified Data.Vector as V
 import qualified System.FilePath.ByteString as P
+import System.PosixCompat.Files (isDirectory)
 import Data.Ord
 import qualified Data.Semigroup as Sem
 import Prelude
@@ -42,11 +43,12 @@ import Logs.Transfer
 import Types.Key
 import Types.TrustLevel
 import Types.FileMatcher
-import Types.ActionItem
+import Types.Availability
 import qualified Limit
 import Messages.JSON (DualDisp(..), ObjectMap(..))
 import Annex.BloomFilter
 import qualified Command.Unused
+import qualified Utility.RawFilePath as R
 
 -- a named computation that produces a statistic
 type Stat = StatState (Maybe (String, StatState String))
@@ -87,12 +89,13 @@ data StatInfo = StatInfo
 	{ presentData :: Maybe KeyInfo
 	, referencedData :: Maybe KeyInfo
 	, repoData :: M.Map UUID KeyInfo
+	, allRepoData :: Maybe KeyInfo
 	, numCopiesStats :: Maybe NumCopiesStats
 	, infoOptions :: InfoOptions
 	}
 
 emptyStatInfo :: InfoOptions -> StatInfo
-emptyStatInfo = StatInfo Nothing Nothing M.empty Nothing
+emptyStatInfo = StatInfo Nothing Nothing M.empty Nothing Nothing
 
 -- a state monad for running Stats in
 type StatState = StateT StatInfo Annex
@@ -108,6 +111,7 @@ data InfoOptions = InfoOptions
 	, bytesOption :: Bool
 	, batchOption :: BatchMode
 	, autoenableOption :: Bool
+	, deadrepositoriesOption :: Bool
 	}
 
 optParser :: CmdParamsDesc -> Parser InfoOptions
@@ -122,6 +126,10 @@ optParser desc = InfoOptions
 		( long "autoenable"
 		<> help "list special remotes that are configured to autoenable"
 		)
+	<*> switch
+		( long "dead-repositories"
+		<> help "list repositories that have been marked as dead"
+		)
 
 seek :: InfoOptions -> CommandSeek
 seek o = case batchOption o of
@@ -133,7 +141,9 @@ start :: InfoOptions -> [String] -> CommandStart
 start o [] = do
 	if autoenableOption o
 		then autoenableInfo
-		else globalInfo o
+		else if deadrepositoriesOption o
+			then deadrepositoriesInfo o
+			else globalInfo o
 	stop
 start o ps = do
 	mapM_ (\p -> itemInfo o (SeekInput [p], p)) ps
@@ -162,31 +172,38 @@ autoenableInfo = showCustom "info" (SeekInput []) $ do
 	showRaw (encodeBS s)
 	return True
 
+deadrepositoriesInfo :: InfoOptions -> Annex ()
+deadrepositoriesInfo o = showCustom "info" (SeekInput []) $ do
+	evalStateT (showStat (repo_list DeadTrusted)) (emptyStatInfo o)
+	return True
+
 itemInfo :: InfoOptions -> (SeekInput, String) -> Annex ()
-itemInfo o (si, p) = ifM (isdir p)
+itemInfo o (si, p) = ifM (isdir (toRawFilePath p))
 	( dirInfo o p si
-	, do
-		v <- Remote.byName' p
-		case v of
-			Right r -> remoteInfo o r si
-			Left _ -> do
-				v' <- Remote.nameToUUID' p
-				case v' of
-					Right u -> uuidInfo o u si
-					Left _ -> do
-						relp <- liftIO $ relPathCwdToFile (toRawFilePath p)
-						lookupKey relp >>= \case
-							Just k -> fileInfo o (fromRawFilePath relp) si k
-							Nothing -> treeishInfo o p si
+	, Remote.byName' p >>= \case
+		Right r -> remoteInfo o r si
+		Left _ -> Remote.nameToUUID' p >>= \case
+			([], _) -> do
+				relp <- liftIO $ relPathCwdToFile (toRawFilePath p)
+				lookupKey relp >>= \case
+					Just k -> fileInfo o (fromRawFilePath relp) si k
+					Nothing -> treeishInfo o p si
+			([u], _) -> uuidInfo o u si
+			(_us, msg) -> noInfo p si msg
 	)
   where
-	isdir = liftIO . catchBoolIO . (isDirectory <$$> getFileStatus)
+	isdir = liftIO . catchBoolIO . (isDirectory <$$> R.getFileStatus)
 
-noInfo :: String -> SeekInput -> Annex ()
-noInfo s si = do
-	showStart "info" (encodeBS s) si
-	showNote $ "not a directory or an annexed file or a treeish or a remote or a uuid"
+noInfo :: String -> SeekInput -> String -> Annex ()
+noInfo s si msg = do
+	-- The string may not really be a file, but use ActionItemTreeFile,
+	-- rather than ActionItemOther to avoid breaking back-compat of
+	-- json output.
+	let ai = ActionItemTreeFile (toRawFilePath s)
+	showStartMessage (StartMessage "info" ai si)
+	showNote (UnquotedString msg)
 	showEndFail
+	Annex.incError
 
 dirInfo :: InfoOptions -> FilePath -> SeekInput -> Annex ()
 dirInfo o dir si = showCustom (unwords ["info", dir]) si $ do
@@ -203,6 +220,7 @@ treeishInfo o t si = do
 	mi <- getTreeStatInfo o (Git.Ref (encodeBS t))
 	case mi of
 		Nothing -> noInfo t si
+			"not a directory or an annexed file or a treeish or a remote or a uuid"
 		Just i -> showCustom (unwords ["info", t]) si $ do
 			stats <- selStats 
 				(tostats (tree_name:tree_fast_stats False)) 
@@ -264,8 +282,10 @@ global_slow_stats =
 	, local_annex_size
 	, known_annex_files True
 	, known_annex_size True
-	, bloom_info
+	, total_annex_size
+	, reposizes_stats_global
 	, backend_usage
+	, bloom_info
 	]
 
 tree_fast_stats :: Bool -> [FilePath -> Stat]
@@ -279,7 +299,7 @@ tree_fast_stats isworktree =
 tree_slow_stats :: [FilePath -> Stat]
 tree_slow_stats =
 	[ const numcopies_stats
-	, const reposizes_stats
+	, const reposizes_stats_tree
 	, const reposizes_total
 	]
 
@@ -296,6 +316,7 @@ remote_fast_stats r = map (\s -> s r)
 	[ remote_name
 	, remote_cost
 	, remote_type
+	, remote_availabile
 	]
 
 uuid_fast_stats :: UUID -> [Stat]
@@ -352,6 +373,10 @@ countRepoList :: Int -> String -> String
 countRepoList _ [] = "0"
 countRepoList n s = show n ++ "\n" ++ beginning s
 
+dispRepoList :: String -> String
+dispRepoList [] = ""
+dispRepoList s = "\n" ++ beginning s
+
 dir_name :: FilePath -> Stat
 dir_name dir = simpleStat "directory" $ pure dir
 
@@ -381,6 +406,11 @@ remote_type :: Remote -> Stat
 remote_type r = simpleStat "type" $ pure $
 	Remote.typename $ Remote.remotetype r
 
+remote_availabile :: Remote -> Stat
+remote_availabile r = simpleStat "available" $ lift $
+	either show (\av -> boolConfig (av /= Unavailable))
+		<$> tryNonAsync (Remote.availability r)
+
 local_annex_keys :: Stat
 local_annex_keys = stat "local annex keys" $ json show $
 	countKeys <$> cachedPresentData
@@ -412,6 +442,12 @@ known_annex_size :: Bool -> Stat
 known_annex_size isworktree = 
 	simpleStat ("size of annexed files in " ++ treeDesc isworktree) $
 		showSizeKeys =<< cachedReferencedData
+
+total_annex_size :: Stat
+total_annex_size = 
+	simpleStat "combined annex size of all repositories" $
+		showSizeKeys . fromMaybe mempty . allRepoData
+		=<< cachedAllRepoData
   
 treeDesc :: Bool -> String
 treeDesc True = "working tree"
@@ -454,15 +490,16 @@ transfer_list = stat desc $ nojson $ lift $ do
 	uuidmap <- Remote.remoteMap id
 	ts <- getTransfers
 	maybeShowJSON $ JSONChunk [(desc, V.fromList $ map (uncurry jsonify) ts)]
+	qp <- coreQuotePath <$> Annex.getGitConfig
 	return $ if null ts
 		then "none"
 		else multiLine $
-			map (uncurry $ line uuidmap) $ sort ts
+			map (uncurry $ line qp uuidmap) $ sort ts
   where
 	desc = "transfers in progress"
-	line uuidmap t i = unwords
+	line qp uuidmap t i = unwords
 		[ fromRawFilePath (formatDirection (transferDirection t)) ++ "ing"
-		, fromRawFilePath $ actionItemDesc $ mkActionItem
+		, fromRawFilePath $ quote qp $ actionItemDesc $ mkActionItem
 			(transferKey t, associatedFile i)
 		, if transferDirection t == Upload then "to" else "from"
 		, maybe (fromUUID $ transferUUID t) Remote.name $
@@ -514,21 +551,29 @@ numcopies_stats = stat "numcopies stats" $ json fmt $
 		. map (\(variance, count) -> "numcopies " ++ variance ++ ": " ++ show count)
 		. V.toList
 
-reposizes_stats :: Stat
-reposizes_stats = stat desc $ nojson $ do
+reposizes_stats_tree :: Stat
+reposizes_stats_tree = reposizes_stats True "repositories containing these files"
+	=<< cachedRepoData
+
+reposizes_stats_global :: Stat
+reposizes_stats_global = reposizes_stats False "annex sizes of repositories" 
+	. repoData =<< cachedAllRepoData
+
+reposizes_stats :: Bool -> String -> M.Map UUID KeyInfo -> Stat
+reposizes_stats count desc m = stat desc $ nojson $ do
 	sizer <- mkSizer
-	l <- map (\(u, kd) -> (u, sizer storageUnits True (sizeKeys kd)))
-		. sortBy (flip (comparing (sizeKeys . snd)))
-		. M.toList
-		<$> cachedRepoData
+	let l = map (\(u, kd) -> (u, sizer storageUnits True (sizeKeys kd))) $
+		sortBy (flip (comparing (sizeKeys . snd))) $
+		M.toList m
 	let maxlen = maximum (map (length . snd) l)
 	descm <- lift Remote.uuidDescriptions
 	-- This also handles json display.
 	s <- lift $ Remote.prettyPrintUUIDsWith (Just "size") desc descm (Just . show) $
 		map (\(u, sz) -> (u, Just $ mkdisp sz maxlen)) l
-	return $ countRepoList (length l) s
+	return $ if count
+		then countRepoList (length l) s
+		else dispRepoList s
   where
-	desc = "repositories containing these files"
 	mkdisp sz maxlen = DualDisp
 		{ dispNormal = lpad maxlen sz
 		, dispJson = sz
@@ -588,11 +633,32 @@ cachedReferencedData = do
 			put s { referencedData = Just v }
 			return v
 
--- currently only available for directory info
+cachedAllRepoData :: StatState StatInfo
+cachedAllRepoData = do
+	s <- get
+	case allRepoData s of
+		Just _ -> return s
+		Nothing -> do
+			matcher <- lift getKeyOnlyMatcher
+			!(d, rd) <- lift $ overLocationLogs (emptyKeyInfo, mempty) $ \k locs (d, rd) -> do
+				ifM (matchOnKey matcher k)
+					( do
+						alivelocs <- snd
+							<$> trustPartition DeadTrusted locs
+						let !d' = addKeyCopies (genericLength alivelocs) k d
+						let !rd' = foldl' (flip (accumrepodata k)) rd alivelocs
+						return (d', rd')
+					, return (d, rd)
+					)
+			let s' = s { allRepoData = Just d, repoData = rd }
+			put s'
+			return s'
+  where
+	accumrepodata k = M.alter (Just . addKey k . fromMaybe emptyKeyInfo)
+
 cachedNumCopiesStats :: StatState (Maybe NumCopiesStats)
 cachedNumCopiesStats = numCopiesStats <$> get
 
--- currently only available for directory info
 cachedRepoData :: StatState (M.Map UUID KeyInfo)
 cachedRepoData = repoData <$> get
 
@@ -603,7 +669,13 @@ getDirStatInfo o dir = do
 	(presentdata, referenceddata, numcopiesstats, repodata) <-
 		Command.Unused.withKeysFilesReferencedIn dir initial
 			(update matcher fast)
-	return $ StatInfo (Just presentdata) (Just referenceddata) repodata (Just numcopiesstats) o
+	return $ StatInfo
+		(Just presentdata)
+		(Just referenceddata)
+		repodata
+		Nothing
+		(Just numcopiesstats)
+		o
   where
 	initial = (emptyKeyInfo, emptyKeyInfo, emptyNumCopiesStats, M.empty)
 	update matcher fast key file vs@(presentdata, referenceddata, numcopiesstats, repodata) =
@@ -639,7 +711,7 @@ getTreeStatInfo o r = do
 	(presentdata, referenceddata, repodata) <- go fast matcher ls initial
 	ifM (liftIO cleanup)
 		( return $ Just $
-			StatInfo (Just presentdata) (Just referenceddata) repodata Nothing o
+			StatInfo (Just presentdata) (Just referenceddata) repodata Nothing Nothing o
 		, return Nothing
 		)
   where
@@ -671,16 +743,19 @@ emptyNumCopiesStats :: NumCopiesStats
 emptyNumCopiesStats = NumCopiesStats M.empty
 
 addKey :: Key -> KeyInfo -> KeyInfo
-addKey key (KeyInfo count size unknownsize backends) =
+addKey = addKeyCopies 1
+
+addKeyCopies :: Integer -> Key -> KeyInfo -> KeyInfo
+addKeyCopies numcopies key (KeyInfo count size unknownsize backends) =
 	KeyInfo count' size' unknownsize' backends'
   where
 	{- All calculations strict to avoid thunks when repeatedly
 	 - applied to many keys. -}
 	!count' = count + 1
 	!backends' = M.insertWith (+) (fromKey keyVariety key) 1 backends
-	!size' = maybe size (+ size) ks
+	!size' = maybe size (\sz -> sz * numcopies + size) ks
 	!unknownsize' = maybe (unknownsize + 1) (const unknownsize) ks
-	ks = fromKey keySize key
+	!ks = fromKey keySize key
 
 updateRepoData :: Key -> [UUID] -> M.Map UUID KeyInfo -> M.Map UUID KeyInfo
 updateRepoData key locs m = m'
