@@ -58,6 +58,7 @@ import Command.Get (getKey')
 import qualified Command.Move
 import qualified Command.Export
 import qualified Command.Import
+import qualified Command.Migrate
 import Annex.Drop
 import Annex.UUID
 import Logs.UUID
@@ -76,8 +77,10 @@ import Annex.TaggedPush
 import Annex.CurrentBranch
 import Annex.Import
 import Annex.CheckIgnore
+import Annex.PidLock
 import Types.FileMatcher
 import Types.GitConfig
+import Types.Availability
 import qualified Database.Export as Export
 import Utility.Bloom
 import Utility.OptParse
@@ -95,7 +98,7 @@ cmd = withAnnexOptions [jobsOption, backendOption] $
 		"synchronize local repository with remotes"
 		(paramRepeating paramRemote) (seek <--< optParser SyncMode)
 
-data OperationMode = SyncMode | PullMode | PushMode | AssistMode
+data OperationMode = SyncMode | PullMode | PushMode | SatisfyMode | AssistMode
 	deriving (Eq, Show)
 
 data SyncOptions = SyncOptions
@@ -143,15 +146,17 @@ optParser mode desc = SyncOptions
 		( metavar desc
 		<> completeRemotes
 		))
-	<*> switch 
-		( long "only-annex"
-		<> short 'a'
-		<> help "do not operate on git branches"
-		)
-	<*> switch 
-		( long "not-only-annex"
-		<> help "operate on git branches as well as annex"
-		)
+	<*> whenmode [SatisfyMode] True 
+		(switch 
+			( long "only-annex"
+			<> short 'a'
+			<> help "do not operate on git branches"
+			))
+	<*> whenmode [SatisfyMode] False 
+		( switch 
+			( long "not-only-annex"
+			<> help "operate on git branches as well as annex"
+			))
 	<*> case mode of
 		SyncMode -> switch
 			( long "commit"
@@ -159,21 +164,25 @@ optParser mode desc = SyncOptions
 			)
 		PushMode -> pure False
 		PullMode -> pure False
+		SatisfyMode -> pure False
 		AssistMode -> pure True
-	<*> unlessmode [SyncMode] True (switch
-		( long "no-commit"
-		<> help "avoid git commit" 
-		))
-	<*> unlessmode [SyncMode, AssistMode] Nothing (optional (strOption
-		( long "message" <> short 'm' <> metavar "MSG"
-		<> help "commit message"
-		)))
+	<*> unlessmode [SyncMode] True 
+		(switch
+			( long "no-commit"
+			<> help "avoid git commit" 
+			))
+	<*> unlessmode [SyncMode, AssistMode] Nothing
+		(optional (strOption
+			( long "message" <> short 'm' <> metavar "MSG"
+			<> help "commit message"
+			)))
 	<*> case mode of
 		SyncMode -> invertableSwitch "pull" True
 			( help "avoid git pulls from remotes" 
 			)
 		PullMode -> pure True
 		PushMode -> pure False
+		SatisfyMode -> pure False
 		AssistMode -> pure True
 	<*> case mode of
 		SyncMode -> invertableSwitch "push" True
@@ -181,31 +190,36 @@ optParser mode desc = SyncOptions
 			)
 		PullMode -> pure False
 		PushMode -> pure True
+		SatisfyMode -> pure False
 		AssistMode -> pure True
-	<*> optional (flag' True 
-		( long "content"
-		<> help "transfer annexed file contents" 
-		))
-	<*> optional (flag' True
-		( long "no-content"
-		<> short 'g'
-		<> help "do not transfer annexed file contents"
-		))
+	<*> whenmode [SatisfyMode] (Just True)
+		(optional (flag' True 
+			( long "content"
+			<> help "transfer annexed file contents" 
+			)))
+	<*> whenmode [SatisfyMode] Nothing
+		(optional (flag' True
+			( long "no-content"
+			<> short 'g'
+			<> help "do not transfer annexed file contents"
+			)))
 	<*> many (strOption
 		( long "content-of"
 		<> short 'C'
 		<> help "transfer contents of annexed files in a given location"
 		<> metavar paramPath
 		))
-	<*> whenmode [PullMode] False (switch
-		( long "cleanup"
-		<> help "remove synced/ branches"
-		))
+	<*> whenmode [PullMode, SatisfyMode] False 
+		(switch
+			( long "cleanup"
+			<> help "remove synced/ branches"
+			))
 	<*> optional parseAllOption
-	<*> whenmode [PushMode] False (invertableSwitch "resolvemerge" True
-		( help "do not automatically resolve merge conflicts"
-		))
-	<*> whenmode [PushMode] False
+	<*> whenmode [PushMode, SatisfyMode] False 
+		(invertableSwitch "resolvemerge" True
+			( help "do not automatically resolve merge conflicts"
+			))
+	<*> whenmode [PushMode, SatisfyMode] False
 		parseUnrelatedHistoriesOption
 	<*> pure mode
   where
@@ -245,17 +259,16 @@ instance DeferredParseClass SyncOptions where
 
 seek :: SyncOptions -> CommandSeek
 seek o = do
-	warnSyncContentTransition o
-	
 	prepMerge
 	
-	startConcurrency transferStages (seek' o)
+	seek' o
 
 seek' :: SyncOptions -> CommandSeek
-seek' o = do
+seek' o = startConcurrency transferStages $ do
 	let withbranch a = a =<< getCurrentBranch
 
 	remotes <- syncRemotes (syncWith o)
+	warnSyncContentTransition o remotes
 	-- Remotes that are git repositories, not (necesarily) special remotes.
 	let gitremotes = filter (Remote.gitSyncableRemoteType . Remote.remotetype) remotes
 	-- Remotes that contain annex object content.
@@ -282,6 +295,10 @@ seek' o = do
 			
 			content <- shouldSyncContent o
 
+			when content $
+				whenM (annexSyncMigrations <$> Annex.getGitConfig) $
+					Command.Migrate.seekDistributedMigrations True
+
 			forM_ (filter isImport contentremotes) $
 				withbranch . importRemote content o
 			forM_ (filter isThirdPartyPopulated contentremotes) $
@@ -340,14 +357,17 @@ mergeConfig mergeunrelated = do
 			else Nothing
 		]
 
-merge :: CurrBranch -> [Git.Merge.MergeConfig] -> SyncOptions -> Git.Branch.CommitMode -> Git.Branch -> Annex Bool
-merge currbranch mergeconfig o commitmode tomerge = do
-	canresolvemerge <- if resolveMergeOverride o
-		then getGitConfigVal annexResolveMerge
-		else return False
-	case currbranch of
-		(Just b, Just adj) -> mergeToAdjustedBranch tomerge (b, adj) mergeconfig canresolvemerge commitmode
-		(b, _) -> autoMergeFrom tomerge b mergeconfig commitmode canresolvemerge
+merge :: CurrBranch -> [Git.Merge.MergeConfig] -> SyncOptions -> Git.Branch.CommitMode -> [Git.Branch] -> Annex Bool
+merge currbranch mergeconfig o commitmode tomergel = 
+	runsGitAnnexChildProcessViaGit $ do
+		canresolvemerge <- if resolveMergeOverride o
+			then getGitConfigVal annexResolveMerge
+			else return False
+		and <$> case currbranch of
+			(Just b, Just adj) -> forM tomergel $ \tomerge ->
+				mergeToAdjustedBranch tomerge (b, adj) mergeconfig canresolvemerge commitmode
+			(b, _) -> forM tomergel $ \tomerge ->
+				autoMergeFrom tomerge b mergeconfig commitmode canresolvemerge
 
 syncBranch :: Git.Branch -> Git.Branch
 syncBranch = Git.Ref.underBase "refs/heads/synced" . origBranch
@@ -358,10 +378,10 @@ origBranch = fromViewBranch . fromAdjustedBranch
 remoteBranch :: Remote -> Git.Ref -> Git.Ref
 remoteBranch remote = Git.Ref.underBase $ "refs/remotes/" ++ Remote.name remote
 
--- Do automatic initialization of remotes when possible when getting remote
--- list.
 syncRemotes :: [String] -> Annex [Remote]
 syncRemotes ps = do
+	-- Do automatic initialization of remotes when possible
+	-- when getting remote list.
 	remotelist <- Remote.remoteList' True
 	available <- filterM (liftIO . getDynamicConfig . remoteAnnexSync . Remote.gitconfig) remotelist
 	syncRemotes' ps available
@@ -376,10 +396,9 @@ syncRemotes' ps available =
 	
 	listed = concat <$> mapM Remote.byNameOrGroup ps
 	
-	good r
-		| Remote.gitSyncableRemoteType (Remote.remotetype r) =
-			Remote.Git.repoAvail =<< Remote.getRepo r
-		| otherwise = return True
+	good r = tryNonAsync (Remote.availability r) >>= return . \case
+		Right Unavailable -> False
+		_ -> True
 	
 	fastest = fromMaybe [] . headMaybe . Remote.byCost
 
@@ -429,46 +448,54 @@ mergeLocal mergeconfig o currbranch = stopUnless (notOnlyAnnex o) $
 mergeLocal' :: [Git.Merge.MergeConfig] -> SyncOptions -> CurrBranch -> CommandStart
 mergeLocal' mergeconfig o currbranch@(Just branch, _) =
 	needMerge currbranch branch >>= \case
-		Nothing -> stop
-		Just syncbranch -> do
-			let ai = ActionItemOther (Just $ UnquotedString $ Git.Ref.describe syncbranch)
+		[] -> stop
+		tomerge -> do
+			let ai = ActionItemOther (Just $ UnquotedString $ unwords $ map Git.Ref.describe tomerge)
 			let si = SeekInput []
 			starting "merge" ai si $
-				next $ merge currbranch mergeconfig o Git.Branch.ManualCommit syncbranch
+				next $ merge currbranch mergeconfig o Git.Branch.ManualCommit tomerge
 mergeLocal' _ _ currbranch@(Nothing, _) = inRepo Git.Branch.currentUnsafe >>= \case
 	Just branch -> needMerge currbranch branch >>= \case
-		Nothing -> stop
-		Just syncbranch -> do
-			let ai = ActionItemOther (Just $ UnquotedString $ Git.Ref.describe syncbranch)
+		[] -> stop
+		tomerge -> do
+			let ai = ActionItemOther (Just $ UnquotedString $ unwords $ map Git.Ref.describe tomerge)
 			let si = SeekInput []
 			starting "merge" ai si $ do
-				warning $ UnquotedString $ "There are no commits yet to branch " ++ Git.fromRef branch ++ ", so cannot merge " ++ Git.fromRef syncbranch ++ " into it."
+				warning $ UnquotedString $ "There are no commits yet to branch " ++ Git.fromRef branch ++ ", so cannot merge " ++ unwords (map Git.fromRef tomerge) ++ " into it."
 				next $ return False
 	Nothing -> stop
 
--- Returns the branch that should be merged, if any.
-needMerge :: CurrBranch -> Git.Branch -> Annex (Maybe Git.Branch)
+-- Returns the branches that should be merged, if any.
+--
+-- Usually this is the sync branch. However, when in an adjusted branch,
+-- it can be either the sync branch or the original branch, or both.
+needMerge :: CurrBranch -> Git.Branch -> Annex [Git.Branch]
 needMerge currbranch headbranch
-	| is_branchView headbranch = return Nothing
-	| otherwise = 
-		ifM (allM id checks)
-			( return (Just syncbranch)
-			, return Nothing
-			)
+	| is_branchView headbranch = return []
+	| otherwise = ifM isBareRepo
+		( return []
+		, do
+			syncbranchret <- usewhen syncbranch syncbranchchecks
+			adjbranchret <- case currbranch of
+				(Just origbranch, Just adj) -> 
+					usewhen origbranch $
+						canMergeToAdjustedBranch origbranch (origbranch, adj)
+				_ -> return []
+			return (syncbranchret++adjbranchret)
+		)
   where
+	usewhen v c = ifM c
+		( return [v]
+		, return []
+		)
 	syncbranch = syncBranch headbranch
-	checks = case currbranch of
-		(Just _, madj) -> 
-			let branch' = maybe headbranch (adjBranch . originalToAdjusted headbranch) madj
-			in 
-				[ not <$> isBareRepo
-				, inRepo (Git.Ref.exists syncbranch)
-				, inRepo (Git.Branch.changed branch' syncbranch)
-				]
-		(Nothing, _) ->
-			[ not <$> isBareRepo
-			, inRepo (Git.Ref.exists syncbranch)
-			]
+	syncbranchchecks = case currbranch of
+		(Just _, madj) -> syncbranchchanged madj
+		(Nothing, _) -> hassyncbranch
+	hassyncbranch = inRepo (Git.Ref.exists syncbranch)
+	syncbranchchanged madj =
+		let branch' = maybe headbranch (adjBranch . originalToAdjusted headbranch) madj
+		in hassyncbranch <&&> inRepo (Git.Branch.changed branch' syncbranch)
 
 pushLocal :: SyncOptions -> CurrBranch -> CommandStart
 pushLocal o b = stopUnless (notOnlyAnnex o) $ do
@@ -564,7 +591,7 @@ importRemote importcontent o remote currbranch
 			let (branch, subdir) = splitRemoteAnnexTrackingBranchSubdir b
 			if canImportKeys remote importcontent
 				then do
-					Command.Import.seekRemote remote branch subdir importcontent (CheckGitIgnore True)
+					Command.Import.seekRemote remote branch subdir importcontent (CheckGitIgnore True) Nothing
 					-- Importing generates a branch
 					-- that is not initially connected
 					-- to the current branch, so allow
@@ -620,8 +647,9 @@ mergeRemote remote currbranch mergeconfig o = ifM isBareRepo
 			mergelisted (tomerge (branchlist (Just branch)))
 	)
   where
-	mergelisted getlist = and <$> 
-		(mapM (merge currbranch mergeconfig o Git.Branch.ManualCommit . remoteBranch remote) =<< getlist)
+	mergelisted getlist =
+		merge currbranch mergeconfig o Git.Branch.ManualCommit
+			=<< map (remoteBranch remote) <$> getlist
 	tomerge = filterM (changed remote)
 	branchlist Nothing = []
 	branchlist (Just branch)
@@ -819,7 +847,7 @@ seekSyncContent o rs currbranch = do
   where
 	seekworktree mvar l bloomfeeder = do
 		let seeker = AnnexedFileSeeker
-			{ startAction = gofile bloomfeeder mvar
+			{ startAction = const $ gofile bloomfeeder mvar
 			, checkContentPresent = Nothing
 			, usesLocationLog = True
 			}
@@ -838,6 +866,7 @@ seekSyncContent o rs currbranch = do
 			SyncMode -> "sync"
 			PullMode -> "pull"
 			PushMode -> "push"
+			SatisfyMode -> "satisfy"
 			AssistMode -> "assist"
 
 	gofile bloom mvar _ f k = 
@@ -875,7 +904,7 @@ seekSyncContent o rs currbranch = do
 syncFile :: SyncOptions -> Either (Maybe (Bloom Key)) (Key -> Annex ()) -> [Remote] -> AssociatedFile -> Key -> Annex Bool
 syncFile o ebloom rs af k = do
 	inhere <- inAnnex k
-	locs <- map Remote.uuid <$> Remote.keyPossibilities k
+	locs <- map Remote.uuid <$> Remote.keyPossibilities (Remote.IncludeIgnored False) k
 	let (have, lack) = partition (\r -> Remote.uuid r `elem` locs) rs
 
 	got <- anyM id =<< handleget have inhere
@@ -910,7 +939,7 @@ syncFile o ebloom rs af k = do
 	return (got || not (null putrs))
   where
 	wantget have inhere = allM id 
-		[ pure (pullOption o)
+		[ pure (pullOption o || operationMode o == SatisfyMode)
 		, pure (not $ null have)
 		, pure (not inhere)
 		, wantGet True (Just k) af
@@ -924,9 +953,9 @@ syncFile o ebloom rs af k = do
 			next $ return True
 
 	wantput r
-		| pushOption o == False = return False
+		| pushOption o == False && operationMode o /= SatisfyMode = return False
 		| Remote.readonly r || remoteAnnexReadOnly (Remote.gitconfig r) = return False
-		| isExport r = return False
+		| isExport r || isImport r = return False
 		| isThirdPartyPopulated r = return False
 		| otherwise = wantGetBy True (Just k) af (Remote.uuid r)
 	handleput lack inhere
@@ -949,6 +978,7 @@ syncFile o ebloom rs af k = do
  - of the branch. (If the branch is not currently checked out, anything
  - imported from the remote will not yet have been merged into it yet and
  - so exporting would delete files from the remote unexpectedly.)
+ - (This is not done in SatifyMode.)
  -
  - Otherwise, transfer any files that were part of a previous export
  - but are not in the remote yet.
@@ -956,15 +986,27 @@ syncFile o ebloom rs af k = do
  - Returns True if any file transfers were made.
  -}
 seekExportContent :: Maybe SyncOptions -> [Remote] -> CurrBranch -> Annex Bool
-seekExportContent o rs (currbranch, _) = or <$> forM rs go
+seekExportContent o rs (mcurrbranch, madj)
+	| null rs = return False
+	| otherwise = do
+		-- Propigate commits from the adjusted branch, so that
+		-- when the remoteAnnexTrackingBranch is set to the parent
+		-- branch, it will be up-to-date.
+		case (mcurrbranch, madj) of
+			(Just currbranch, Just adj) ->
+				propigateAdjustedCommits currbranch adj
+			_ -> noop
+		or <$> forM rs go
   where
 	go r
+		| maybe False (\o' -> operationMode o' == SatisfyMode) o =
+			case remoteAnnexTrackingBranch (Remote.gitconfig r) of
+				Nothing -> return False
+				Just _ -> withdb r $ \db ->
+					cannotupdateexport r db Nothing False
 		| not (maybe True pushOption o) = return False
 		| not (remoteAnnexPush (Remote.gitconfig r)) = return False
-		| otherwise = bracket
-			(Export.openDb (Remote.uuid r))
-			Export.closeDb
-			(\db -> Export.writeLockDbWhile db (go' r db))
+		| otherwise = withdb r (go' r)
 	go' r db = case remoteAnnexTrackingBranch (Remote.gitconfig r) of
 		Nothing -> cannotupdateexport r db Nothing True
 		Just b -> do
@@ -975,7 +1017,7 @@ seekExportContent o rs (currbranch, _) = or <$> forM rs go
 				Nothing -> id
 			mcurrtree <- maybe (pure Nothing)
 				(inRepo . Git.Ref.tree . addsubdir)
-				currbranch
+				mcurrbranch
 			mtbcommitsha <- Command.Export.getExportCommit r b
 			case (mtree, mcurrtree, mtbcommitsha) of
 				(Just tree, Just currtree, Just _)
@@ -983,16 +1025,23 @@ seekExportContent o rs (currbranch, _) = or <$> forM rs go
 						filteredtree <- Command.Export.filterExport r tree
 						Command.Export.changeExport r db filteredtree
 						Command.Export.fillExport r db filteredtree mtbcommitsha
-					| otherwise -> cannotupdateexport r db (Just b) False
-				_ -> cannotupdateexport r db (Just b) True
+					| otherwise -> cannotupdateexport r db Nothing False
+				(Nothing, _, _) -> cannotupdateexport r db (Just (Git.fromRef b ++ " does not exist")) True
+				(_, Nothing, _) -> cannotupdateexport r db (Just "no branch is currently checked out") True
+				(_, _, Nothing) -> cannotupdateexport r db (Just "tracking branch name is not valid") True
 	
-	cannotupdateexport r db mtb showwarning = do
+	withdb r a = bracket
+		(Export.openDb (Remote.uuid r))
+		Export.closeDb
+		(\db -> Export.writeLockDbWhile db (a db))
+	
+	cannotupdateexport r db mreason showwarning = do
 		exported <- getExport (Remote.uuid r)
 		when showwarning $
-			maybe noop (warncannotupdateexport r mtb exported) currbranch
+			maybe noop (warncannotupdateexport r mreason exported) mcurrbranch
 		fillexistingexport r db (exportedTreeishes exported) Nothing
 	
-	warncannotupdateexport r mtb exported currb = case mtb of
+	warncannotupdateexport r mreason exported currb = case mreason of
 		Nothing -> inRepo (Git.Ref.tree currb) >>= \case
 			Just currt | not (any (== currt) (exportedTreeishes exported)) ->
 				showLongNote $ UnquotedString $ unwords
@@ -1002,9 +1051,9 @@ seekExportContent o rs (currbranch, _) = or <$> forM rs go
 					, "(Set " ++ gitconfig ++ " to enable it.)"
 					]
 			_ -> noop
-		Just b -> showLongNote $ UnquotedString $ unwords
+		Just reason -> showLongNote $ UnquotedString $ unwords
 			[ notupdating
-			, "because " ++ Git.fromRef b ++ " does not exist."
+			, "because " ++ reason ++ "."
 			, "(As configured by " ++ gitconfig ++ ")"
 			]
 	  where
@@ -1056,6 +1105,7 @@ cleanupRemote remote (Just b, _) =
 shouldSyncContent :: SyncOptions -> Annex Bool
 shouldSyncContent o
 	| fromMaybe False (noContentOption o) = pure False
+	| operationMode o == SatisfyMode = pure True
 	-- For git-annex pull and git-annex push and git-annex assist,
 	-- annex.syncontent defaults to True unless set
 	| operationMode o /= SyncMode = annexsynccontent True
@@ -1070,23 +1120,28 @@ shouldSyncContent o
 			HasGitConfig (Just c) -> return c
 			_ -> return d
 
--- Transition started May 2023, should wait until that has been in a Debian
--- stable release before completing the transition.
-warnSyncContentTransition :: SyncOptions -> Annex ()
-warnSyncContentTransition o
+-- See doc/todo/finish_sync_content_transition.mdwn
+warnSyncContentTransition :: SyncOptions -> [Remote] -> Annex ()
+warnSyncContentTransition o remotes
 	| operationMode o /= SyncMode = noop
 	| isJust (noContentOption o) || isJust (contentOption o) = noop
 	| not (null (contentOfOption o)) = noop
 	| otherwise = getGitConfigVal' annexSyncContent >>= \case
 		HasGlobalConfig (Just _) -> noop
 		HasGitConfig (Just _) -> noop
-		_ -> showwarning
+		_ -> do
+			m <- preferredContentMap
+			hereu <- getUUID
+			when (any (`M.member` m) (hereu:map Remote.uuid remotes)) $
+				showwarning
   where
 	showwarning = earlyWarning $
-		"git-annex sync will change default behavior to operate on"
-		<> " --content in a future version of git-annex. Recommend"
-		<> " you explicitly use --no-content (or -g) to prepare for"
-		<> " that change. (Or you can configure annex.synccontent)"
+		"git-annex sync will change default behavior in the future to"
+		<> " send content to repositories that have"
+		<> " preferred content configured. If you do not want this to"
+		<> " send any content, use --no-content (or -g)"
+		<> " to prepare for that change."
+		<> " (Or you can configure annex.synccontent)"
 
 notOnlyAnnex :: SyncOptions -> Annex Bool
 notOnlyAnnex o = not <$> onlyAnnex o
