@@ -38,11 +38,13 @@ import Annex.InodeSentinal
 import qualified Data.Map as M
 import qualified Data.ByteString as S
 import qualified Data.ByteString.Char8 as S8
+import qualified Data.Text as T
 import Data.Char
 
 cmd :: Command
-cmd = command "unused" SectionMaintenance "look for unused file content"
-	paramNothing (seek <$$> optParser)
+cmd = withAnnexOptions [jsonOptions] $
+	command "unused" SectionMaintenance "look for unused file content"
+		paramNothing (seek <$$> optParser)
 
 data UnusedOptions = UnusedOptions
 	{ fromRemote :: Maybe RemoteName
@@ -73,7 +75,7 @@ start o = do
 		Just "." -> (".", checkUnused refspec)
 		Just "here" -> (".", checkUnused refspec)
 		Just n -> (n, checkRemoteUnused n refspec)
-	starting "unused" (ActionItemOther (Just name)) (SeekInput []) perform
+	starting "unused" (ActionItemOther (Just (UnquotedString name))) (SeekInput []) perform
 
 checkUnused :: RefSpec -> CommandPerform
 checkUnused refspec = chain 0
@@ -105,12 +107,16 @@ checkRemoteUnused remotename refspec = go =<< Remote.nameToUUID remotename
 		Just ks -> excludeReferenced refspec ks
 		Nothing -> giveup "This repository is read-only."
 
-check :: FilePath -> ([(Int, Key)] -> String) -> Annex [Key] -> Int -> Annex Int
-check file msg a c = do
+check :: String -> ([(Int, Key)] -> String) -> Annex [Key] -> Int -> Annex Int
+check fileprefix msg a c = do
 	l <- a
 	let unusedlist = number c l
-	unless (null l) $ showLongNote $ msg unusedlist
-	updateUnusedLog (toRawFilePath file) (M.fromList unusedlist)
+	unless (null l) $
+		showLongNote $ UnquotedString $ msg unusedlist
+	maybeAddJSONField
+		((if null fileprefix then "unused" else fileprefix) ++ "-list")
+		(M.fromList $ map (\(n,  k) -> (T.pack (show n), serializeKey k)) unusedlist)
+	updateUnusedLog (toRawFilePath fileprefix) (M.fromList unusedlist)
 	return $ c + length l
 
 number :: Int -> [a] -> [(Int, a)]
@@ -249,7 +255,7 @@ withKeysReferencedDiffGitRefs refspec a = do
  - differ from those referenced in the index. -}
 withKeysReferencedDiffGitRef :: (Key -> Annex ()) -> Git.Ref -> Annex ()
 withKeysReferencedDiffGitRef a ref = do
-	showAction $ "checking " ++ Git.Ref.describe ref
+	showAction $ UnquotedString $ "checking " ++ Git.Ref.describe ref
 	withKeysReferencedDiff a
 		(inRepo $ DiffTree.diffIndex ref)
 		DiffTree.srcsha
@@ -321,12 +327,12 @@ unusedSpec m spec
 
 {- Seek action for unused content. Finds the number in the maps, and
  - calls one of 3 actions, depending on the type of unused file. -}
-startUnused :: String
-	-> (Key -> CommandPerform)
-	-> (Key -> CommandPerform) 
-	-> (Key -> CommandPerform)
+startUnused
+	:: (Int -> Key -> CommandStart)
+	-> (Int -> Key -> CommandStart) 
+	-> (Int -> Key -> CommandStart)
 	-> UnusedMaps -> Int -> CommandStart
-startUnused message unused badunused tmpunused maps n = search
+startUnused unused badunused tmpunused maps n = search
 	[ (unusedMap maps, unused)
 	, (unusedBadMap maps, badunused)
 	, (unusedTmpMap maps, tmpunused)
@@ -336,7 +342,4 @@ startUnused message unused badunused tmpunused maps n = search
 	search ((m, a):rest) =
 		case M.lookup n m of
 			Nothing -> search rest
-			Just key -> starting message
-				(ActionItemOther $ Just $ show n)
-				(SeekInput [])
-				(a key)
+			Just key -> a n key
