@@ -5,6 +5,8 @@
  - Licensed under the GNU AGPL version 3 or higher.
  -}
 
+{-# LANGUAGE OverloadedStrings #-}
+
 module Command.View where
 
 import Command
@@ -118,7 +120,7 @@ checkoutViewBranch view madj mkbranch = do
 		forM_ l (removeemptydir top)
 		liftIO $ void cleanup
 		unlessM (liftIO $ doesDirectoryExist here) $ do
-			showLongNote (cwdmissing (fromRawFilePath top))
+			showLongNote $ UnquotedString $ cwdmissing (fromRawFilePath top)
 	return ok
   where
 	removeemptydir top d = do
