@@ -17,7 +17,6 @@ import Remote.Web (getWebUrls)
 import Annex.UUID
 import qualified Utility.Format
 import qualified Command.Find
-import Types.ActionItem
 
 import qualified Data.Map as M
 import qualified Data.Vector as V
@@ -52,7 +51,7 @@ seek :: WhereisOptions -> CommandSeek
 seek o = do
 	m <- remoteMap id
 	let seeker = AnnexedFileSeeker
-		{ startAction = start o m
+		{ startAction = const $ start o m
 		, checkContentPresent = Nothing
 		, usesLocationLog = True
 		}
@@ -65,7 +64,7 @@ seek o = do
 		Batch fmt -> batchOnly (keyOptions o) (whereisFiles o) $
 			batchAnnexed fmt seeker (startKeys o m)
   where
-	ww = WarnUnmatchLsFiles
+	ww = WarnUnmatchLsFiles "whereis"
 
 start :: WhereisOptions -> M.Map UUID Remote -> SeekInput -> RawFilePath -> Key -> CommandStart
 start o remotemap si file key = 
@@ -88,12 +87,14 @@ perform o remotemap key ai = do
 	case formatOption o of
 		Nothing -> do
 			let num = length safelocations
-			showNote $ show num ++ " " ++ copiesplural num
+			showNote $ UnquotedString $ show num ++ " " ++ copiesplural num
 			pp <- ppwhereis "whereis" safelocations urls
-			unless (null safelocations) $ showLongNote pp
+			unless (null safelocations) $
+				showLongNote (UnquotedString pp)
 			pp' <- ppwhereis "untrusted" untrustedlocations urls
-			unless (null untrustedlocations) $ showLongNote $ untrustedheader ++ pp'
-		
+			unless (null untrustedlocations) $
+				showLongNote $ UnquotedString $
+					untrustedheader ++ pp'
 			mapM_ (showRemoteUrls remotemap) urls
 		Just formatter -> liftIO $ do
 			let vs = Command.Find.formatVars key
@@ -143,7 +144,7 @@ getRemoteUrls key remote
 		Just w -> tryNonAsync (w key) >>= \case
 			Right l -> pure l
 			Left e -> do
-				warning $ unwords
+				warning $ UnquotedString $ unwords
 					[ "unable to query remote"
 					, name remote
 					, "for urls:"
@@ -161,6 +162,6 @@ showRemoteUrls :: M.Map UUID Remote -> (UUID, [URLString]) -> Annex ()
 showRemoteUrls remotemap (uu, us)
 	| null us = noop
 	| otherwise = case M.lookup uu remotemap of
-		Just r -> showLongNote $ 
+		Just r -> showLongNote $ UnquotedString $
 			unlines $ map (\u -> name r ++ ": " ++ u) us 
 		Nothing -> noop
