@@ -81,8 +81,10 @@ get = do
 			}
 		r <- Git.Config.read $ (newFrom loc)
 			{ gitDirSpecifiedExplicitly = True }
-		return $ if Git.Config.isBare r
-			then r { location = (location r) { worktree = Nothing } }
+		return $ if fromMaybe False (Git.Config.isBare r)
+			then case location r of
+				loc'@(Local {}) -> r { location = loc' { worktree = Nothing } }
+				_ -> r
 			else r
 	configure Nothing Nothing = giveup "Not in a git repository."
 
