@@ -78,7 +78,7 @@ doMerge hashhandle ch differ repo streamer = do
 	go [] = noop
 	go (info:file:rest) = mergeFile info file hashhandle ch >>=
 		maybe (go rest) (\l -> streamer l >> go rest)
-	go (_:[]) = error $ "parse error " ++ show differ
+	go (_:[]) = giveup $ "parse error " ++ show differ
 
 {- Given an info line from a git raw diff, and the filename, generates
  - a line suitable for update-index that union merges the two sides of the
@@ -108,9 +108,9 @@ mergeFile info file hashhandle h = case S8.words info of
  -}
 calcMerge :: [(Ref, [L8.ByteString])] -> Either Ref [L8.ByteString]
 calcMerge shacontents
-	| null reuseable = Right new
-	| otherwise = Left $ fst $ Prelude.head reuseable
+	| null reusable = Right new
+	| otherwise = Left $ fst $ Prelude.head reusable
   where
-	reuseable = filter (\c -> sorteduniq (snd c) == new) shacontents
+	reusable = filter (\c -> sorteduniq (snd c) == new) shacontents
 	new = sorteduniq $ concat $ map snd shacontents
 	sorteduniq = S.toList . S.fromList
