@@ -2,7 +2,7 @@
  -
  - Copyright 2011, 2013 Joey Hess <id@joeyh.name>
  -
- - Licensed under the GNU GPL version 3 or higher.
+ - Licensed under the GNU AGPL version 3 or higher.
  -}
 
 {-# OPTIONS_GHC -fno-warn-tabs #-}
@@ -14,7 +14,7 @@ module Git.Version (
 	GitVersion,
 ) where
 
-import Common
+import Utility.Process
 import Utility.DottedVersion
 
 type GitVersion = DottedVersion
