@@ -2,7 +2,7 @@
  -
  - Copyright 2015 Joey Hess <id@joeyh.name>
  -
- - Licensed under the GNU GPL version 3 or higher.
+ - Licensed under the GNU AGPL version 3 or higher.
  -}
 
 module Logs.Difference.Pure (
@@ -20,7 +20,7 @@ import Logs.UUIDBased
 
 parseDifferencesLog :: L.ByteString -> (M.Map UUID Differences)
 parseDifferencesLog = simpleMap
-	. parseLog (readDifferences . decodeBS <$> A.takeByteString)
+	. parseLogOld (readDifferences . decodeBS <$> A.takeByteString)
 
 -- The sum of all recorded differences, across all UUIDs.
 allDifferences :: M.Map UUID Differences -> Differences
