@@ -5,7 +5,7 @@
  - Licensed under the GNU AGPL version 3 or higher.
  -}
 
-{-# LANGUAGE OverloadedStrings, ScopedTypeVariables #-}
+{-# LANGUAGE OverloadedStrings, ScopedTypeVariables, CPP #-}
 
 module Messages (
 	showStartMessage,
@@ -50,6 +50,7 @@ module Messages (
 	outputMessage,
 	withMessageState,
 	MessageState,
+	explain,
 	prompt,
 	mkPrompter,
 	sanitizeTopLevelExceptionMessages,
@@ -264,6 +265,12 @@ setupConsole = do
 	 - a file or a pipe. -}
 	hSetBuffering stdout LineBuffering
 	hSetBuffering stderr LineBuffering
+#ifdef mingw32_HOST_OS
+	{- Avoid outputting CR at end of line on Windows. git commands do
+	 - not ouput CR there. -}
+	hSetNewlineMode stdout noNewlineTranslation
+	hSetNewlineMode stderr noNewlineTranslation
+#endif
 
 enableDebugOutput :: Annex ()
 enableDebugOutput = do
@@ -299,6 +306,16 @@ jsonOutputEnabled = withMessageState $ \s -> return $
 		JSONOutput _ -> True
 		_ -> False
 
+explain :: ActionItem -> Maybe StringContainingQuotedPath -> Annex ()
+explain ai (Just msg) = do
+	rd <- Annex.getRead id
+	let d = actionItemDesc ai
+	let msg' = "[ " <> (if d == mempty then "" else (d <> " ")) <> msg <> " ]\n"
+	if Annex.explainenabled rd
+		then outputMessage JSON.none id msg'
+		else fastDebug' rd "Messages.explain" (decodeBS (noquote msg'))
+explain _ _ = return ()
+
 {- Prevents any concurrent console access while running an action, so
  - that the action is the only thing using the console, and can eg prompt
  - the user.
