@@ -32,9 +32,10 @@ import Control.Monad
 import Control.Monad.Free
 import Control.Monad.Free.TH
 import Control.Monad.Catch
-import System.FilePath
 import System.Exit (ExitCode(..))
 import System.IO
+import qualified System.FilePath.ByteString as P
+import qualified Data.ByteString as B
 import qualified Data.ByteString.Lazy as L
 import Data.Char
 import Control.Applicative
@@ -179,11 +180,11 @@ instance Proto.Serializable ProtoAssociatedFile where
 			| isSpace c = "%"
 			| otherwise = [c]
 	
-	deserialize s = case fromRawFilePath $ fromInternalGitPath $ toRawFilePath $ deesc [] s of
-		[] -> Just $ ProtoAssociatedFile $ AssociatedFile Nothing
+	deserialize s = case fromInternalGitPath $ toRawFilePath $ deesc [] s of
 		f
-			| isRelative f -> Just $ ProtoAssociatedFile $ 
-				AssociatedFile $ Just $ toRawFilePath f
+			| B.null f -> Just $ ProtoAssociatedFile $ AssociatedFile Nothing
+			| P.isRelative f -> Just $ ProtoAssociatedFile $ 
+				AssociatedFile $ Just f
 			| otherwise -> Nothing
 	  where
 	  	deesc b [] = reverse b
@@ -262,7 +263,7 @@ data LocalF c
 	-- so, terminate the protocol connection.
 	--
 	-- If the validity check is provided and fails, the content was
-	-- changed while it was being sent, so verificiation of the
+	-- changed while it was being sent, so verification of the
 	-- received content should be forced.
 	--
 	-- Note: The ByteString may not contain the entire remaining content
@@ -386,7 +387,7 @@ serverLoop a = do
 		-- gives up, since it's not clear what state the client
 		-- is in, and so not possible to recover.
 		Just (ERROR _) -> return Nothing
-		-- When the client sends an unparseable message, the server
+		-- When the client sends an unparsable message, the server
 		-- responds with an error message, and loops. This allows
 		-- expanding the protocol with new messages.
 		Nothing -> do
