@@ -23,7 +23,7 @@ import Types.CleanupActions
 import Messages.Progress
 import Utility.Metered
 import Utility.Tmp
-import Utility.Url (parseURIPortable)
+import Utility.Url.Parse
 import Backend.URL
 import Annex.Perms
 import Annex.Tmp
@@ -47,7 +47,7 @@ remote = RemoteType
 	, enumerate = list
 	, generate = gen
 	, configParser = mkRemoteConfigParser []
-	, setup = error "not supported"
+	, setup = giveup "not supported"
 	, exportSupported = exportUnsupported
 	, importSupported = importUnsupported
 	, thirdPartyPopulated = False
@@ -88,7 +88,7 @@ gen r _ rc gc rs = do
 		, readonly = True
 		, appendonly = False
 		, untrustworthy = False
-		, availability = GloballyAvailable
+		, availability = pure GloballyAvailable
 		, remotetype = remote
 		, mkUnavailable = return Nothing
 		, getInfo = return []
