@@ -54,6 +54,7 @@ import Types.Availability (Availability(..))
 import Types.Key
 import Git.Types
 import Utility.Url (URLString)
+import Utility.Url.Parse
 import qualified Utility.SimpleProtocol as Proto
 
 import Control.Concurrent.STM
@@ -109,6 +110,7 @@ supportedExtensionList :: ExtensionList
 supportedExtensionList = ExtensionList
 	[ "INFO"
 	, "GETGITREMOTENAME"
+	, "UNAVAILABLERESPONSE"
 	, asyncExtension
 	]
 
@@ -415,7 +417,7 @@ newtype JobId = JobId Integer
 	deriving (Eq, Ord, Show)
 
 supportedProtocolVersions :: [ProtocolVersion]
-supportedProtocolVersions = [1]
+supportedProtocolVersions = [1, 2]
 
 instance Proto.Serializable JobId where
 	serialize (JobId n) = show n
@@ -446,9 +448,11 @@ instance Proto.Serializable Size where
 instance Proto.Serializable Availability where
 	serialize GloballyAvailable = "GLOBAL"
 	serialize LocallyAvailable = "LOCAL"
+	serialize Unavailable = "UNAVAILABLE"
 
 	deserialize "GLOBAL" = Just GloballyAvailable
 	deserialize "LOCAL" = Just LocallyAvailable
+	deserialize "UNAVAILABLE" = Just Unavailable
 	deserialize _ = Nothing
 
 instance Proto.Serializable [(URLString, Size, FilePath)] where
@@ -462,7 +466,7 @@ instance Proto.Serializable [(URLString, Size, FilePath)] where
 
 instance Proto.Serializable URI where
 	serialize = show
-	deserialize = parseURI
+	deserialize = parseURIPortable
 
 instance Proto.Serializable ExportLocation where
 	serialize = fromRawFilePath . fromExportLocation
