@@ -103,7 +103,7 @@ gen r u rc gc rs = do
 			, readonly = False
 			, appendonly = False
 			, untrustworthy = False
-			, availability = GloballyAvailable
+			, availability = pure GloballyAvailable
 			, remotetype = remote
 			, mkUnavailable = return Nothing
 			, getInfo = includeCredsInfo c (AWS.creds u) $
