@@ -1,6 +1,6 @@
 {- git-annex test suite
  -
- - Copyright 2010-2022 oey Hess <id@joeyh.name>
+ - Copyright 2010-2023 Joey Hess <id@joeyh.name>
  -
  - Licensed under the GNU AGPL version 3 or higher.
  -}
@@ -26,11 +26,12 @@ import Control.Concurrent.STM hiding (check)
 
 import Common
 import CmdLine.GitAnnex.Options
+import qualified Utility.RawFilePath as R
+import Data.String
 
 import qualified Utility.ShellEscape
-import qualified Utility.RawFilePath as R
 import qualified Annex
-import qualified Git.Filename
+import qualified Git.Quote
 import qualified Git.Types
 import qualified Git.Ref
 import qualified Git.LsTree
@@ -77,12 +78,12 @@ import qualified Utility.Hash
 import qualified Utility.Scheduled
 import qualified Utility.Scheduled.QuickCheck
 import qualified Utility.HumanTime
-import qualified Utility.Base64
 import qualified Utility.Tmp.Dir
 import qualified Utility.FileSystemEncoding
 import qualified Utility.Aeson
 import qualified Utility.CopyFile
 import qualified Utility.MoveFile
+import qualified Utility.StatelessOpenPGP
 import qualified Types.Remote
 #ifndef mingw32_HOST_OS
 import qualified Remote.Helper.Encryptable
@@ -128,7 +129,7 @@ runner :: TestOptions -> IO ()
 runner opts = parallelTestRunner opts tests
 
 tests :: Int -> Bool -> Bool -> TestOptions -> [TestTree]
-tests n crippledfilesystem adjustedbranchok opts = 
+tests numparts crippledfilesystem adjustedbranchok opts = 
 	properties 
 		: withTestMode remotetestmode testRemotes
 		: concatMap mkrepotests testmodes
@@ -147,11 +148,11 @@ tests n crippledfilesystem adjustedbranchok opts =
 		| otherwise = Nothing
 	mkrepotests (d, te) = map 
 		(\uts -> withTestMode te uts)
-		(repoTests d n)
+		(repoTests d numparts)
 
 properties :: TestTree
 properties = localOption (QuickCheckTests 1000) $ testGroup "QuickCheck" $
-	[ testProperty "prop_encode_decode_roundtrip" Git.Filename.prop_encode_decode_roundtrip
+	[ testProperty "prop_quote_unquote_roundtrip" Git.Quote.prop_quote_unquote_roundtrip
 	, testProperty "prop_encode_c_decode_c_roundtrip" Utility.Format.prop_encode_c_decode_c_roundtrip
 	, testProperty "prop_isomorphic_key_encode" Key.prop_isomorphic_key_encode
 	, testProperty "prop_isomorphic_shellEscape" Utility.ShellEscape.prop_isomorphic_shellEscape
@@ -184,7 +185,6 @@ properties = localOption (QuickCheckTests 1000) $ testGroup "QuickCheck" $
 	, testProperty "prop_viewPath_roundtrips" Annex.View.prop_viewPath_roundtrips
 	, testProperty "prop_view_roundtrips" Annex.View.prop_view_roundtrips
 	, testProperty "prop_viewedFile_rountrips" Annex.View.ViewedFile.prop_viewedFile_roundtrips
-	, testProperty "prop_b64_roundtrips" Utility.Base64.prop_b64_roundtrips
 	, testProperty "prop_standardGroups_parse" Logs.PreferredContent.prop_standardGroups_parse
 	] ++ map (uncurry testProperty) combos
   where
@@ -239,7 +239,7 @@ testRemote testvariants remotetype setupremote =
 		innewrepo $ do
 			git_annex "init" [reponame, "--quiet"] "init"
 			setupremote remotename
-			r <- annexeval $ either error return 
+			r <- annexeval $ either giveup return 
 				=<< Remote.byName' remotename
 			cache <- Command.TestRemote.newRemoteVariantCache
 			unavailr <- annexeval $ Types.Remote.mkUnavailable r
@@ -282,8 +282,8 @@ repoTests note numparts = map mk $ sep
 	, testCase "readonly remote" test_readonly_remote
 	, testCase "ignore deleted files" test_ignore_deleted_files
 	, testCase "metadata" test_metadata
-	, testCase "export_import" test_export_import
-	, testCase "export_import_subdir" test_export_import_subdir
+	, testCase "export and import" test_export_import
+	, testCase "export and import of subdir" test_export_import_subdir
 	, testCase "shared clone" test_shared_clone
 	, testCase "log" test_log
 	, testCase "view" test_view
@@ -349,11 +349,13 @@ repoTests note numparts = map mk $ sep
 	, testCase "rsync remote" test_rsync_remote
 	, testCase "bup remote" test_bup_remote
 	, testCase "borg remote" test_borg_remote
-	, testCase "crypto" test_crypto
+	, testCase "gpg crypto" test_gpg_crypto
+	, testCase "sop crypto" test_sop_crypto
 	, testCase "preferred content" test_preferred_content
 	, testCase "required_content" test_required_content
 	, testCase "add subdirs" test_add_subdirs
 	, testCase "addurl" test_addurl
+	, testCase "repair" test_repair
 	]
   where
 	mk l = testGroup groupname (initTests : map adddep l)
@@ -439,7 +441,7 @@ test_readonly_remote =
 				git_annex "get" [annexedfile] "get failed in first repo"
 			make_readonly r1
 			indir r2 $ do
-				git_annex "sync" ["r1", "--no-push"] "sync with readonly repo"
+				git_annex "sync" ["r1", "--no-push", "--no-content"] "sync with readonly repo"
 				git_annex "get" [annexedfile, "--from", "r1"] "get from readonly repo"
 				git "remote" ["rm", "origin"] "remote rm"
 				git_annex "drop" [annexedfile] "drop vs readonly repo"
@@ -511,7 +513,7 @@ test_magic = intmpclonerepo $ do
 	writeFile "text" "test\n" 
 	git_annex "add" ["binary", "text"]
 		"git-annex add with mimeencoding in largefiles"
-	git_annex "sync" []
+	git_annex "sync" ["--no-content"]
 		"git-annex sync"
 	(isJust <$> annexeval (Annex.CatFile.catKeyFile (encodeBS "binary")))
 		@? "binary file not added to annex despite mimeencoding config"
@@ -706,7 +708,7 @@ test_move_numcopies = intmpclonerepo $ do
 	annexed_notpresent annexedfile
 	inmainrepo $ annexed_present annexedfile
 	git_annex "get" [annexedfile] "get of file"
-	git_annex_shouldfail "move" ["--from", "origin", annexedfile] "move of file --from remote that violates numcopies setting not allowd"
+	git_annex_shouldfail "move" ["--from", "origin", annexedfile] "move of file --from remote that violates numcopies setting not allowed"
 	git_annex_shouldfail "move" ["--to", "origin", annexedfile] "move of file --to remote that violates numcopies setting not allowed"
 
 test_copy :: Assertion
@@ -1051,13 +1053,13 @@ test_unused = intmpclonerepo $ do
 	checkunused [] "after rm"
 	-- commit the rm, and when on an adjusted branch, sync it back to
 	-- the master branch
-	git_annex "sync" ["--no-push", "--no-pull"] "git-annex sync"
+	git_annex "sync" ["--no-push", "--no-pull", "--no-content"] "git-annex sync"
 	checkunused [] "after commit"
 	-- unused checks origin/master; once it's gone it is really unused
 	git "remote" ["rm", "origin"] "git remote rm origin"
 	checkunused [annexedfilekey] "after origin branches are gone"
 	git "rm" ["-fq", sha1annexedfile] "git rm"
-	git_annex "sync" ["--no-push", "--no-pull"] "git-annex sync"
+	git_annex "sync" ["--no-push", "--no-pull", "--no-content"] "git-annex sync"
 	checkunused [annexedfilekey, sha1annexedfilekey] "after rm sha1annexedfile"
 
 	-- good opportunity to test dropkey also
@@ -1163,6 +1165,12 @@ test_info :: Assertion
 test_info = intmpclonerepo $ do
 	isjson
 	readonly_query isjson
+	-- When presented with an input that it does not support,
+	-- info does not stop but processes subsequent inputs too.
+	git_annex'' (const True)
+		(sha1annexedfile `isInfixOf`)
+		"info"
+		[annexedfile, "dnefile", sha1annexedfile] Nothing "info"
   where
 	isjson = do
 		json <- BU8.fromString <$> git_annex_output "info" ["--json"]
@@ -1177,7 +1185,7 @@ test_version = intmpclonerepo $ do
 
 test_sync :: Assertion
 test_sync = intmpclonerepo $ do
-	git_annex "sync" [] "sync"
+	git_annex "sync" ["--no-content"] "sync"
 	{- Regression test for bug fixed in
 	 - 039e83ed5d1a11fd562cce55b8429c840d72443e, where a present
 	 - wanted file was dropped. -}
@@ -1233,11 +1241,11 @@ test_union_merge_regression =
 					git_annex "get" [annexedfile] "get"
 					git "remote" ["rm", "origin"] "remote rm"
 				forM_ [r3, r2, r1] $ \r -> indir r $
-					git_annex "sync" [] ("sync in " ++ r)
+					git_annex "sync" ["--no-content"] ("sync in " ++ r)
 				forM_ [r3, r2] $ \r -> indir r $
 					git_annex "drop" ["--force", annexedfile] ("drop in " ++ r)
 				indir r1 $ do
-					git_annex "sync" [] "sync in r1"
+					git_annex "sync" ["--no-content"] "sync in r1"
 					git_annex_expectoutput "find" ["--in", "r3"] []
 					{- This was the bug. The sync
 					 - mangled location log data and it
@@ -1263,7 +1271,7 @@ test_conflict_resolution_movein_regression = withtmpclonerepo $ \r1 ->
 		{- Sync twice in r1 so it gets the conflict resolution
 		 - update from r2 -}
 		forM_ [r1, r2, r1] $ \r -> indir r $
-			git_annex "sync" ["--force"] ("sync in " ++ rname r)
+			git_annex "sync" ["--force", "--no-content"] ("sync in " ++ rname r)
 		{- After the sync, it should be possible to get all
 		 - files. This includes both sides of the conflict,
 		 - although the filenames are not easily predictable.
@@ -1283,15 +1291,15 @@ test_conflict_resolution =
 				disconnectOrigin
 				writecontent conflictor "conflictor1"
 				add_annex conflictor "add conflicter"
-				git_annex "sync" [] "sync in r1"
+				git_annex "sync" ["--no-content"] "sync in r1"
 			indir r2 $ do
 				disconnectOrigin
 				writecontent conflictor "conflictor2"
 				add_annex conflictor "add conflicter"
-				git_annex "sync" [] "sync in r2"
+				git_annex "sync" ["--no-content"] "sync in r2"
 			pair r1 r2
 			forM_ [r1,r2,r1] $ \r -> indir r $
-				git_annex "sync" [] "sync"
+				git_annex "sync" ["--no-content"] "sync"
 			checkmerge "r1" r1
 			checkmerge "r2" r2
   where
@@ -1316,19 +1324,19 @@ test_conflict_resolution_adjusted_branch =
 				disconnectOrigin
 				writecontent conflictor "conflictor1"
 				add_annex conflictor "add conflicter"
-				git_annex "sync" [] "sync in r1"
+				git_annex "sync" ["--no-content"] "sync in r1"
 			indir r2 $ do
 				disconnectOrigin
 				writecontent conflictor "conflictor2"
 				add_annex conflictor "add conflicter"
-				git_annex "sync" [] "sync in r2"
+				git_annex "sync" ["--no-content"] "sync in r2"
 				-- We might be in an adjusted branch
 				-- already, when eg on a crippled
 				-- filesystem. So, --force it.
 				git_annex "adjust" ["--unlock", "--force"] "adjust"
 			pair r1 r2
 			forM_ [r1,r2,r1] $ \r -> indir r $
-				git_annex "sync" [] "sync"
+				git_annex "sync" ["--no-content"] "sync"
 			checkmerge "r1" r1
 			checkmerge "r2" r2
   where
@@ -1357,17 +1365,17 @@ test_mixed_conflict_resolution = do
 				disconnectOrigin
 				writecontent conflictor "conflictor"
 				add_annex conflictor "add conflicter"
-				git_annex "sync" [] "sync in r1"
+				git_annex "sync" ["--no-content"] "sync in r1"
 			indir r2 $ do
 				disconnectOrigin
 				createDirectory conflictor
 				writecontent subfile "subfile"
 				add_annex conflictor "add conflicter"
-				git_annex "sync" [] "sync in r2"
+				git_annex "sync" ["--no-content"] "sync in r2"
 			pair r1 r2
 			let l = if inr1 then [r1, r2] else [r2, r1]
 			forM_ l $ \r -> indir r $
-				git_annex "sync" [] "sync in mixed conflict"
+				git_annex "sync" ["--no-content"] "sync in mixed conflict"
 			checkmerge "r1" r1
 			checkmerge "r2" r2
 	conflictor = "conflictor"
@@ -1399,12 +1407,12 @@ test_remove_conflict_resolution = do
 				disconnectOrigin
 				writecontent conflictor "conflictor"
 				add_annex conflictor "add conflicter"
-				git_annex "sync" [] "sync in r1"
+				git_annex "sync" ["--no-content"] "sync in r1"
 			indir r2 $
 				disconnectOrigin
 			pair r1 r2
 			indir r2 $ do
-				git_annex "sync" [] "sync in r2"
+				git_annex "sync" ["--no-content"] "sync in r2"
 				git_annex "get" [conflictor] "get conflictor"
 				git_annex "unlock" [conflictor] "unlock conflictor"
 				writecontent conflictor "newconflictor"
@@ -1412,7 +1420,7 @@ test_remove_conflict_resolution = do
 				removeWhenExistsWith R.removeLink (toRawFilePath conflictor)
 			let l = if inr1 then [r1, r2, r1] else [r2, r1, r2]
 			forM_ l $ \r -> indir r $
-				git_annex "sync" [] "sync"
+				git_annex "sync" ["--no-content"] "sync"
 			checkmerge "r1" r1
 			checkmerge "r2" r2
 	conflictor = "conflictor"
@@ -1439,7 +1447,7 @@ test_nonannexed_file_conflict_resolution = do
 				disconnectOrigin
 				writecontent conflictor "conflictor"
 				add_annex conflictor "add conflicter"
-				git_annex "sync" [] "sync in r1"
+				git_annex "sync" ["--no-content"] "sync in r1"
 			indir r2 $ do
 				disconnectOrigin
 				writecontent conflictor nonannexed_content
@@ -1448,11 +1456,11 @@ test_nonannexed_file_conflict_resolution = do
 					, "exclude=" ++ ingitfile ++ " and exclude=" ++ conflictor
 					] "git config annex.largefiles"
 				git "add" [conflictor] "git add conflictor"
-				git_annex "sync" [] "sync in r2"
+				git_annex "sync" ["--no-content"] "sync in r2"
 			pair r1 r2
 			let l = if inr1 then [r1, r2] else [r2, r1]
 			forM_ l $ \r -> indir r $
-				git_annex "sync" [] "sync"
+				git_annex "sync" ["--no-content"] "sync"
 			checkmerge "r1" r1
 			checkmerge "r2" r2
 	conflictor = "conflictor"
@@ -1489,16 +1497,16 @@ test_nonannexed_symlink_conflict_resolution = do
 					disconnectOrigin
 					writecontent conflictor "conflictor"
 					add_annex conflictor "add conflicter"
-					git_annex "sync" [] "sync in r1"
+					git_annex "sync" ["--no-content"] "sync in r1"
 				indir r2 $ do
 					disconnectOrigin
-					createSymbolicLink symlinktarget "conflictor"
+					R.createSymbolicLink (toRawFilePath symlinktarget) (toRawFilePath "conflictor")
 					git "add" [conflictor] "git add conflictor"
-					git_annex "sync" [] "sync in r2"
+					git_annex "sync" ["--no-content"] "sync in r2"
 				pair r1 r2
 				let l = if inr1 then [r1, r2] else [r2, r1]
 				forM_ l $ \r -> indir r $
-					git_annex "sync" [] "sync"
+					git_annex "sync" ["--no-content"] "sync"
 				checkmerge "r1" r1
 				checkmerge "r2" r2
 	conflictor = "conflictor"
@@ -1512,8 +1520,8 @@ test_nonannexed_symlink_conflict_resolution = do
 		length v == 1
 			@? (what ++ " too many variant files in: " ++ show v)
 		conflictor `elem` l @? (what ++ " conflictor file missing in: " ++ show l)
-		s <- catchMaybeIO (readSymbolicLink (d </> conflictor))
-		s == Just symlinktarget
+		s <- catchMaybeIO (R.readSymbolicLink (toRawFilePath (d </> conflictor)))
+		s == Just (toRawFilePath symlinktarget)
 			@? (what ++ " wrong target for nonannexed symlink: " ++ show s)
 
 {- Check merge conflict resolution when there is a local file,
@@ -1537,14 +1545,14 @@ test_uncommitted_conflict_resolution = do
 				createDirectoryIfMissing True (fromRawFilePath (parentDir (toRawFilePath remoteconflictor)))
 				writecontent remoteconflictor annexedcontent
 				add_annex conflictor "add remoteconflicter"
-				git_annex "sync" [] "sync in r1"
+				git_annex "sync" ["--no-content"] "sync in r1"
 			indir r2 $ do
 				disconnectOrigin
 				writecontent conflictor localcontent
 			pair r1 r2
 			-- this case is intentionally not handled
 			-- since the user can recover on their own easily
-			indir r2 $ git_annex_shouldfail "sync" [] "sync should not succeed"
+			indir r2 $ git_annex_shouldfail "sync" ["--no-content"] "sync should not succeed"
 	conflictor = "conflictor"
 	localcontent = "local"
 	annexedcontent = "annexed"
@@ -1560,18 +1568,18 @@ test_conflict_resolution_symlink_bit = unlessM (hasUnlockedFiles <$> getTestMode
 				indir r1 $ do
 					writecontent conflictor "conflictor"
 					git_annex "add" [conflictor] "add conflicter"
-					git_annex "sync" [] "sync in r1"
+					git_annex "sync" ["--no-content"] "sync in r1"
 					check_is_link conflictor "r1"
 				indir r2 $ do
 					createDirectory conflictor
 					writecontent (conflictor </> "subfile") "subfile"
 					git_annex "add" [conflictor] "add conflicter"
-					git_annex "sync" [] "sync in r2"
+					git_annex "sync" ["--no-content"] "sync in r2"
 					check_is_link (conflictor </> "subfile") "r2"
 				indir r3 $ do
 					writecontent conflictor "conflictor"
 					git_annex "add" [conflictor] "add conflicter"
-					git_annex "sync" [] "sync in r1"
+					git_annex "sync" ["--no-content"] "sync in r1"
 					check_is_link (conflictor </> "subfile") "r3"
   where
 	conflictor = "conflictor"
@@ -1592,16 +1600,16 @@ test_mixed_lock_conflict_resolution =
 				disconnectOrigin
 				writecontent conflictor "conflictor"
 				git_annex "add" [conflictor] "add conflicter"
-				git_annex "sync" [] "sync in r1"
+				git_annex "sync" ["--no-content"] "sync in r1"
 			indir r2 $ do
 				disconnectOrigin
 				writecontent conflictor "conflictor"
 				git_annex "add" [conflictor] "add conflicter"
 				git_annex "unlock" [conflictor] "unlock conflicter"
-				git_annex "sync" [] "sync in r2"
+				git_annex "sync" ["--no-content"] "sync in r2"
 			pair r1 r2
 			forM_ [r1,r2,r1] $ \r -> indir r $
-				git_annex "sync" [] "sync"
+				git_annex "sync" ["--no-content"] "sync"
 			checkmerge "r1" r1
 			checkmerge "r2" r2
   where
@@ -1638,9 +1646,9 @@ test_adjusted_branch_merge_regression = do
 		git_annex "adjust" ["--unlock", "--force"] "adjust"
 		writecontent conflictor "conflictor"
 		git_annex "add" [conflictor] "add conflicter"
-		git_annex "sync" [] "sync"
+		git_annex "sync" ["--no-content"] "sync"
 	checkmerge what d = indir d $ whensupported $ do
-		git_annex "sync" [] ("sync should not work in " ++ what)
+		git_annex "sync" ["--no-content"] ("sync should not work in " ++ what)
 		l <- getDirectoryContents "."
 		conflictor `elem` l
 			@? ("conflictor not present after merge in " ++ what)
@@ -1661,11 +1669,11 @@ test_adjusted_branch_subtree_regression =
 			createDirectoryIfMissing True "a/b/c"
 			writecontent "a/b/c/d" "foo"
 			git_annex "add" ["a/b/c"] "add a/b/c"
-			git_annex "sync" [] "sync"
+			git_annex "sync" ["--no-content"] "sync"
 			createDirectoryIfMissing True "a/b/x"
 			writecontent "a/b/x/y" "foo"
 			git_annex "add" ["a/b/x"] "add a/b/x"
-			git_annex "sync" [] "sync"
+			git_annex "sync" ["--no-content"] "sync"
 			git "checkout" [origbranch] "git checkout"
 			doesFileExist "a/b/x/y" @? ("a/b/x/y missing from master after adjusted branch sync")
 
@@ -1682,7 +1690,7 @@ test_uninit = intmpclonerepo $ do
 	git_annex "get" [] "get"
 	annexed_present annexedfile
 	-- any exit status is accepted; does abnormal exit
-	git_annex'' (const True) "uninit" [] Nothing "uninit"
+	git_annex'' (const True) (const True) "uninit" [] Nothing "uninit"
 	checkregularfile annexedfile
 	doesDirectoryExist ".git" @? ".git vanished in uninit"
 
@@ -1797,19 +1805,19 @@ test_borg_remote = when BuildInfo.borg $ do
 	borgdirparent <- fromRawFilePath <$> (absPath . toRawFilePath =<< tmprepodir)
 	let borgdir = borgdirparent </> "borgrepo"
 	intmpclonerepo $ do
-		testProcess "borg" ["init", borgdir, "-e", "none"] Nothing (== True) "borg init"
-		testProcess "borg" ["create", borgdir++"::backup1", "."] Nothing (== True) "borg create"
+		testProcess "borg" ["init", borgdir, "-e", "none"] Nothing (== True) (const True) "borg init"
+		testProcess "borg" ["create", borgdir++"::backup1", "."] Nothing (== True) (const True) "borg create"
 
 		git_annex "initremote" (words $ "borg type=borg borgrepo="++borgdir) "initremote"
-		git_annex "sync" ["borg"] "sync borg"
+		git_annex "sync" ["--no-content", "borg"] "sync borg"
 		git_annex_expectoutput "find" ["--in=borg"] []
 
 		git_annex "get" [annexedfile] "get of file"
 		annexed_present annexedfile
 		git_annex_expectoutput "find" ["--in=borg"] []
 		
-		testProcess "borg" ["create", borgdir++"::backup2", "."] Nothing (== True) "borg create"
-		git_annex "sync" ["borg"] "sync borg after getting file"
+		testProcess "borg" ["create", borgdir++"::backup2", "."] Nothing (== True) (const True) "borg create"
+		git_annex "sync" ["--no-content", "borg"] "sync borg after getting file"
 		git_annex_expectoutput "find" ["--in=borg"] [annexedfile]
 
 		git "remote" ["rm", "origin"] "remote rm"
@@ -1819,10 +1827,29 @@ test_borg_remote = when BuildInfo.borg $ do
 		git_annex "drop" [annexedfile] "drop from borg (appendonly)"
 		git_annex "get" [annexedfile, "--from=borg"] "get from borg"
 
+-- To test Stateless OpenPGP, annex.shared-sop-command has to be set using
+-- the --test-git-config option.
+test_sop_crypto :: Assertion
+test_sop_crypto = do
+	gc <- testGitConfig . testOptions <$> getTestMode
+	case filter (\(k, _) -> k == ck) gc of
+		[] -> noop
+		((_, sopcmd):_) -> go $ 
+			Utility.StatelessOpenPGP.SOPCmd $
+				Git.Types.fromConfigValue sopcmd
+  where
+	ck = fromString "annex.shared-sop-command"
+	pw = fromString "testpassword"
+	v = fromString "somevalue"
+	unarmored = Utility.StatelessOpenPGP.Armoring False
+	go sopcmd = do
+		Utility.StatelessOpenPGP.test_encrypt_decrypt_Symmetric sopcmd sopcmd pw unarmored v
+			@? "sop command roundtrips symmetric encryption"
+
 -- gpg is not a build dependency, so only test when it's available
-test_crypto :: Assertion
+test_gpg_crypto :: Assertion
 #ifndef mingw32_HOST_OS
-test_crypto = do
+test_gpg_crypto = do
 	testscheme "shared"
 	testscheme "hybrid"
 	testscheme "pubkey"
@@ -1831,7 +1858,7 @@ test_crypto = do
 	testscheme scheme = Utility.Tmp.Dir.withTmpDir "gpgtmp" $ \gpgtmp -> do
 		-- Use the system temp directory as gpg temp directory because 
 		-- it needs to be able to store the agent socket there,
-		-- which can be problimatic when testing some filesystems.
+		-- which can be problematic when testing some filesystems.
 		absgpgtmp <- fromRawFilePath <$> absPath (toRawFilePath gpgtmp)
 		res <- testscheme' scheme absgpgtmp
 		-- gpg may still be running and would prevent
@@ -1914,7 +1941,7 @@ test_crypto = do
 			Annex.Locations.keyPaths .
 			Crypto.encryptKey Types.Crypto.HmacSha1 cipher
 #else
-test_crypto = putStrLn "gpg testing not implemented on Windows"
+test_gpg_crypto = putStrLn "gpg testing not implemented on Windows"
 #endif
 
 test_add_subdirs :: Assertion
@@ -1926,7 +1953,7 @@ test_add_subdirs = intmpclonerepo $ do
 	{- Regression test for Windows bug where symlinks were not
 	 - calculated correctly for files in subdirs. -}
 	unlessM (hasUnlockedFiles <$> getTestMode) $ do
-		git_annex "sync" [] "sync"
+		git_annex "sync" ["--no-content"] "sync"
 		l <- annexeval $ Utility.FileSystemEncoding.decodeBL
 			<$> Annex.CatFile.catObject (Git.Types.Ref (encodeBS "HEAD:dir/foo"))
 		"../.git/annex/" `isPrefixOf` l @? ("symlink from subdir to .git/annex is wrong: " ++ l)
@@ -2002,7 +2029,7 @@ test_export_import = intmpclonerepo $ do
 	-- When on an adjusted branch, this updates the master branch
 	-- to match it, which is necessary since the master branch is going
 	-- to be exported.
-	commitchanges = git_annex "sync" ["--no-pull", "--no-push"] "sync"
+	commitchanges = git_annex "sync" ["--no-pull", "--no-push", "--no-content"] "sync"
 
 test_export_import_subdir :: Assertion
 test_export_import_subdir = intmpclonerepo $ do
@@ -2018,7 +2045,7 @@ test_export_import_subdir = intmpclonerepo $ do
 	-- When on an adjusted branch, this updates the master branch
 	-- to match it, which is necessary since the master branch is going
 	-- to be exported.
-	git_annex "sync" ["--no-pull", "--no-push"] "sync"
+	git_annex "sync" ["--no-pull", "--no-push", "--no-content"] "sync"
 
 	-- Run three times because there was a bug that took a couple
 	-- of runs to lead to the wrong tree being written to the remote
@@ -2060,12 +2087,12 @@ test_transition_propagation_reversion =
 				disconnectOrigin
 				writecontent wormannexedfile $ content wormannexedfile
 				git_annex "add" [wormannexedfile, "--backend=WORM"] "add"
-				git_annex "sync" [] "sync"
+				git_annex "sync" ["--no-content"] "sync"
 			indir r2 $ do
 				disconnectOrigin
-				git_annex "sync" [] "sync"
+				git_annex "sync" ["--no-content"] "sync"
 			indir r1 $ do
-				git_annex "sync" [] "sync"
+				git_annex "sync" ["--no-content"] "sync"
 			indir r2 $ do
 				git_annex "get" [wormannexedfile] "get"
 				git_annex "drop" [wormannexedfile] "drop"
@@ -2073,15 +2100,20 @@ test_transition_propagation_reversion =
 				git_annex "drop" [wormannexedfile] "drop"
 			indir r1 $ do
 				git_annex "drop" ["--force", wormannexedfile] "drop"
-				git_annex "sync" [] "sync"
+				git_annex "sync" ["--no-content"] "sync"
 				git_annex "forget" ["--force"] "forget"
-				git_annex "sync" [] "sync"
+				git_annex "sync" ["--no-content"] "sync"
 				emptylog
 			indir r2 $ do
-				git_annex "sync" [] "sync"
+				git_annex "sync" ["--no-content"] "sync"
 				emptylog
 			indir r1 $ do
-				git_annex "sync" [] "sync"
+				git_annex "sync" ["--no-content"] "sync"
 				emptylog
   where
 	emptylog = git_annex_expectoutput "log" [wormannexedfile] []
+
+test_repair :: Assertion
+test_repair = intmpclonerepo $
+	-- Simply running repair used to fail on Windows.
+	git_annex "repair" [] "repair"
