@@ -1,6 +1,6 @@
 {- git-annex test suite framework
  -
- - Copyright 2010-2022 Joey Hess <id@joeyh.name>
+ - Copyright 2010-2023 Joey Hess <id@joeyh.name>
  -
  - Licensed under the GNU AGPL version 3 or higher.
  -}
@@ -23,12 +23,15 @@ import Control.Concurrent.STM
 import System.Environment (getArgs)
 import System.Console.Concurrent
 import System.Console.ANSI
+import Data.Time.Clock
 import GHC.Conc
 import System.IO.Unsafe (unsafePerformIO)
+import System.PosixCompat.Files (isSymbolicLink, isRegularFile, fileMode, unionFileModes, ownerWriteMode)
 
 import Common
 import Types.Test
 import Types.Concurrency
+import qualified Utility.RawFilePath as R
 
 import qualified Annex
 import qualified Annex.UUID
@@ -61,33 +64,36 @@ import qualified Utility.Exception
 import qualified Utility.ThreadScheduler
 import qualified Utility.Tmp.Dir
 import qualified Utility.Metered
+import qualified Utility.HumanTime
 import qualified Command.Uninit
 
 -- Run a process. The output and stderr is captured, and is only
 -- displayed if the process does not return the expected value.
 --
 -- In debug mode, the output is allowed to pass through.
-testProcess :: String -> [String] -> Maybe [(String, String)] -> (Bool -> Bool) -> String -> Assertion
-testProcess command params environ expectedret faildesc = do
+-- So the output does not get checked in debug mode.
+testProcess :: String -> [String] -> Maybe [(String, String)] -> (Bool -> Bool) -> (String -> Bool) -> String -> Assertion
+testProcess command params environ expectedret expectedtranscript faildesc = do
 	let p = (proc command params) { env = environ }
 	debug <- testDebug . testOptions <$> getTestMode
 	if debug
 		then do
 			ret <- withCreateProcess p $ \_ _ _ pid ->
 				waitForProcess pid
-			(expectedret (ret == ExitSuccess)) @? (faildesc ++ " failed")
+			(expectedret (ret == ExitSuccess)) @? (faildesc ++ " failed with unexpected exit code")
 		else do
 			(transcript, ret) <- Utility.Process.Transcript.processTranscript' p Nothing
-			(expectedret ret) @? (faildesc ++ " failed (transcript follows)\n" ++ transcript)
+			(expectedret ret) @? (faildesc ++ " failed with unexpected exit code (transcript follows)\n" ++ transcript)
+			(expectedtranscript transcript) @? (faildesc ++ " failed with unexpected output (transcript follows)\n" ++ transcript)
 
 -- Run git. (Do not use to run git-annex as the one being tested
 -- may not be in path.)
 git :: String -> [String] -> String -> Assertion
-git command params = testProcess "git" (command:params) Nothing (== True)
+git command params = testProcess "git" (command:params) Nothing (== True) (const True)
 
 -- For when git is expected to fail.
 git_shouldfail :: String -> [String] -> String -> Assertion
-git_shouldfail command params = testProcess "git" (command:params) Nothing (== False)
+git_shouldfail command params = testProcess "git" (command:params) Nothing (== False) (const True)
 
 -- Run git-annex.
 git_annex :: String -> [String] -> String -> Assertion
@@ -95,23 +101,23 @@ git_annex command params faildesc = git_annex' command params Nothing faildesc
 
 -- Runs git-annex with some environment.
 git_annex' :: String -> [String] -> Maybe [(String, String)] -> String -> Assertion
-git_annex' = git_annex'' (== True)
+git_annex' = git_annex'' (== True) (const True)
 
 -- For when git-annex is expected to fail.
 git_annex_shouldfail :: String -> [String] -> String -> Assertion
 git_annex_shouldfail command params faildesc = git_annex_shouldfail' command params Nothing faildesc
 
 git_annex_shouldfail' :: String -> [String] -> Maybe [(String, String)] -> String -> Assertion
-git_annex_shouldfail' = git_annex'' (== False)
+git_annex_shouldfail' = git_annex'' (== False) (const True)
 
-git_annex'' :: (Bool -> Bool) -> String -> [String] -> Maybe [(String, String)] -> String -> Assertion
-git_annex'' expectedret command params environ faildesc = do
+git_annex'' :: (Bool -> Bool) -> (String -> Bool) -> String -> [String] -> Maybe [(String, String)] -> String -> Assertion
+git_annex'' expectedret expectedtranscript command params environ faildesc = do
 	pp <- Annex.Path.programPath
 	debug <- testDebug . testOptions <$> getTestMode
 	let params' = if debug
 		then "--debug":params
 		else params
-	testProcess pp (command:params') environ expectedret faildesc
+	testProcess pp (command:params') environ expectedret expectedtranscript faildesc
 
 {- Runs git-annex and returns its standard output. -}
 git_annex_output :: String -> [String] -> IO String
@@ -349,24 +355,24 @@ checklink f = ifM (annexeval Config.crippledFileSystem)
 	( (isJust <$> annexeval (Annex.Link.getAnnexLinkTarget (toRawFilePath f)))
 		@? f ++ " is not a (crippled) symlink"
 	, do
-		s <- getSymbolicLinkStatus f
+		s <- R.getSymbolicLinkStatus (toRawFilePath f)
 		isSymbolicLink s @? f ++ " is not a symlink"
 	)
 
 checkregularfile :: FilePath -> Assertion
 checkregularfile f = do
-	s <- getSymbolicLinkStatus f
+	s <- R.getSymbolicLinkStatus (toRawFilePath f)
 	isRegularFile s @? f ++ " is not a normal file"
 	return ()
 
 checkdoesnotexist :: FilePath -> Assertion
 checkdoesnotexist f = 
-	(either (const True) (const False) <$> Utility.Exception.tryIO (getSymbolicLinkStatus f))
+	(either (const True) (const False) <$> Utility.Exception.tryIO (R.getSymbolicLinkStatus (toRawFilePath f)))
 		@? f ++ " exists unexpectedly"
 
 checkexists :: FilePath -> Assertion
 checkexists f = 
-	(either (const False) (const True) <$> Utility.Exception.tryIO (getSymbolicLinkStatus f))
+	(either (const False) (const True) <$> Utility.Exception.tryIO (R.getSymbolicLinkStatus (toRawFilePath f)))
 		@? f ++ " does not exist"
 
 checkcontent :: FilePath -> Assertion
@@ -379,14 +385,14 @@ checkunwritable f = do
 	-- Look at permissions bits rather than trying to write or
 	-- using fileAccess because if run as root, any file can be
 	-- modified despite permissions.
-	s <- getFileStatus f
+	s <- R.getFileStatus (toRawFilePath f)
 	let mode = fileMode s
 	when (mode == mode `unionFileModes` ownerWriteMode) $
 		assertFailure $ "able to modify annexed file's " ++ f ++ " content"
 
 checkwritable :: FilePath -> Assertion
 checkwritable f = do
-	s <- getFileStatus f
+	s <- R.getFileStatus (toRawFilePath f)
 	let mode = fileMode s
 	unless (mode == mode `unionFileModes` ownerWriteMode) $
 		assertFailure $ "unable to modify " ++ f
@@ -720,7 +726,7 @@ runFakeSsh (_host:cmd:[]) =
 		\_ _ _ pid -> exitWith =<< waitForProcess pid
 runFakeSsh ps = error $ "fake ssh option parse error: " ++ show ps
 
-{- Tests each TestTree in parallel, and exits with succcess/failure.
+{- Tests each TestTree in parallel, and exits with success/failure.
  -
  - Tasty supports parallel tests, but this does not use it, because
  - many tests need to be run in test repos, and chdir would not be 
@@ -767,7 +773,28 @@ parallelTestRunner' numjobs opts mkts
 				r <- a n
 				worker (r:rs) nvar a
 	
-	go Nothing = withConcurrentOutput $ do
+	summarizeresults a = do
+		starttime <- getCurrentTime
+		(numts, exitcodes) <- a
+		duration <- Utility.HumanTime.durationSince starttime
+		case nub (filter (/= ExitSuccess) (concat exitcodes)) of
+			[] -> do
+				putStrLn ""
+				putStrLn $ "All tests succeeded. (Ran "
+					++ show numts
+					++ " test groups in " 
+					++ Utility.HumanTime.fromDuration duration
+					++ ")"
+				exitSuccess
+			[ExitFailure 1] -> do
+				putStrLn "  (Failures above could be due to a bug in git-annex, or an incompatibility"
+				putStrLn "   with utilities, such as git, installed on this system.)"
+				exitFailure
+			_ -> do
+				putStrLn $ "  Test subprocesses exited with unexpected exit codes: " ++ show (concat exitcodes)
+				exitFailure
+
+	go Nothing = summarizeresults $ withConcurrentOutput $ do
 		ensuredir tmpdir
 		crippledfilesystem <- fst <$> Annex.Init.probeCrippledFileSystem'
 			(toRawFilePath tmpdir)
@@ -798,15 +825,7 @@ parallelTestRunner' numjobs opts mkts
 		exitcodes <- forConcurrently [1..numjobs] $ \_ -> 
 			worker [] nvar runone
 		unless (keepFailuresOption opts) finalCleanup
-		case nub (filter (/= ExitSuccess) (concat exitcodes)) of
-			[] -> exitSuccess
-			[ExitFailure 1] -> do
-				putStrLn "  (Failures above could be due to a bug in git-annex, or an incompatibility"
-				putStrLn "   with utilities, such as git, installed on this system.)"
-				exitFailure
-			_ -> do
-				putStrLn $ "  Test subprocesses exited with unexpected exit codes: " ++ show (concat exitcodes)
-				exitFailure
+		return (length ts, exitcodes)
 	go (Just subenvval) = case readish subenvval of
 		Nothing -> error ("Bad " ++ subenv)
 		Just (n, crippledfilesystem, adjustedbranchok) -> setTestEnv $ do
