@@ -5,12 +5,17 @@
  - Licensed under the GNU AGPL version 3 or higher.
  -}
 
+{-# LANGUAGE OverloadedStrings #-}
+
 module Upgrade.V0 where
 
 import Annex.Common
 import Types.Upgrade
 import Annex.Content
 import qualified Upgrade.V1
+import qualified Utility.RawFilePath as R
+
+import System.PosixCompat.Files (isRegularFile)
 
 upgrade :: Annex UpgradeResult
 upgrade = do
@@ -46,7 +51,8 @@ getKeysPresent0 dir = ifM (liftIO $ doesDirectoryExist dir)
   where
 	present d = do
 		result <- tryIO $
-			getFileStatus $ dir ++ "/" ++ takeFileName d
+			R.getFileStatus $ toRawFilePath $
+				dir ++ "/" ++ takeFileName d
 		case result of
 			Right s -> return $ isRegularFile s
 			Left _ -> return False
