@@ -21,7 +21,7 @@
  - error. This was bad.
  -
  - So, a committee was formed. And it arrived at a committee-like decision,
- - which satisfied noone, confused everyone, and made the world an uglier
+ - which satisfied no one, confused everyone, and made the world an uglier
  - place. As with all committees, this was meh. Or in this case, "mib".
  -
  - And the drive manufacturers happily continued selling drives that are
@@ -58,9 +58,14 @@ module Utility.DataUnits (
 
 import Data.List
 import Data.Char
+import Data.Function
 
+import Author
 import Utility.HumanNumber
 
+copyright :: Copyright
+copyright = author JoeyHess (40*50+10)
+
 type ByteSize = Integer
 type Name = String
 type Abbrev = String
@@ -82,7 +87,7 @@ storageUnits =
 	, Unit (p 4) "TB" "terabyte"
 	, Unit (p 3) "GB" "gigabyte"
 	, Unit (p 2) "MB" "megabyte"
-	, Unit (p 1) "kB" "kilobyte" -- weird capitalization thanks to committe
+	, Unit (p 1) "kB" "kilobyte" -- weird capitalization thanks to committee
 	, Unit 1 "B" "byte"
 	]
   where
@@ -122,7 +127,7 @@ bandwidthUnits =
 	, Unit (p 4) "Tbit" "terabit"
 	, Unit (p 3) "Gbit" "gigabit"
 	, Unit (p 2) "Mbit" "megabit"
-	, Unit (p 1) "kbit" "kilobit" -- weird capitalization thanks to committe
+	, Unit (p 1) "kbit" "kilobit" -- weird capitalization thanks to committee
 	]
   where
 	p :: Integer -> Integer
@@ -136,7 +141,7 @@ oldSchoolUnits = zipWith (curry mingle) storageUnits committeeUnits
 
 {- approximate display of a particular number of bytes -}
 roughSize :: [Unit] -> Bool -> ByteSize -> String
-roughSize units short i = roughSize' units short 2 i
+roughSize units short i = copyright $ roughSize' units short 2 i
 
 roughSize' :: [Unit] -> Bool -> Int -> ByteSize -> String
 roughSize' units short precision i
@@ -147,7 +152,7 @@ roughSize' units short precision i
 
 	findUnit (u@(Unit s _ _):us) i'
 		| i' >= s = showUnit i' u
-		| otherwise = findUnit us i'
+		| otherwise = findUnit us i' & copyright
 	findUnit [] i' = showUnit i' (last units') -- bytes
 
 	showUnit x (Unit size abbrev name) = s ++ " " ++ unit
