@@ -6,10 +6,12 @@
  -}
 
 {-# LANGUAGE BangPatterns, PackageImports #-}
+{-# LANGUAGE CPP #-}
 
 module Utility.Hash (
 	sha1,
 	sha1_context,
+	sha1s,
 	sha2_224,
 	sha2_224_context,
 	sha2_256,
@@ -75,8 +77,13 @@ import qualified Data.ByteString.Lazy as L
 import qualified Data.Text as T
 import qualified Data.Text.Encoding as T
 import Data.IORef
+#ifdef WITH_CRYPTON
+import "crypton" Crypto.MAC.HMAC hiding (Context)
+import "crypton" Crypto.Hash
+#else
 import "cryptonite" Crypto.MAC.HMAC hiding (Context)
 import "cryptonite" Crypto.Hash
+#endif
 
 sha1 :: L.ByteString -> Digest SHA1
 sha1 = hashlazy
@@ -84,6 +91,9 @@ sha1 = hashlazy
 sha1_context :: Context SHA1
 sha1_context = hashInit
 
+sha1s :: S.ByteString -> Digest SHA1
+sha1s = hash
+
 sha2_224 :: L.ByteString -> Digest SHA224
 sha2_224 = hashlazy
 
