@@ -18,12 +18,9 @@ module Utility.LockFile.Windows (
 import System.Win32.Types
 import System.Win32.File
 import Control.Concurrent
-import qualified Data.ByteString as B
-import qualified System.FilePath.Windows.ByteString as P
 
+import Utility.Path.Windows
 import Utility.FileSystemEncoding
-import Utility.Split
-import Utility.Path.AbsRel
 
 type LockFile = RawFilePath
 
@@ -60,7 +57,7 @@ lockExclusive = openLock fILE_SHARE_NONE
  -}
 openLock :: ShareMode -> LockFile -> IO (Maybe LockHandle)
 openLock sharemode f = do
-	f' <- convertToNativeNamespace f
+	f' <- convertToWindowsNativeNamespace f
 #if MIN_VERSION_Win32(2,13,4)
 	r <- tryNonAsync $ createFile_NoRetry f' gENERIC_READ sharemode 
 		security_attributes oPEN_ALWAYS fILE_ATTRIBUTE_NORMAL
@@ -79,37 +76,11 @@ openLock sharemode f = do
   where
 	security_attributes = maybePtr Nothing
 
-{- Convert a filepath to use Windows's native namespace.
- - This avoids filesystem length limits.
- -
- - This is similar to the way base converts filenames on windows,
- - but as that is implemented in C (create_device_name) and not
- - exported, it cannot be used here. Several edge cases are not handled,
- - including network shares and dos short paths. 
- -}
-convertToNativeNamespace :: RawFilePath -> IO RawFilePath
-convertToNativeNamespace f
-	| win32_dev_namespace `B.isPrefixOf` f = return f
-	| win32_file_namespace `B.isPrefixOf` f = return f
-	| nt_device_namespace `B.isPrefixOf` f = return f
-	| otherwise = do
-		-- Make absolute because any '.' and '..' in the path
-		-- will not be resolved once it's converted.
-		p <- absPath f
-		-- Normalize slashes.
-		let p' = P.normalise p
-		return (win32_file_namespace <> p')
-  where
- 
-	win32_dev_namespace = "\\\\.\\"
-	win32_file_namespace = "\\\\?\\"
-	nt_device_namespace = "\\Device\\"
-
 dropLock :: LockHandle -> IO ()
 dropLock = closeHandle
 
 {- If the initial lock fails, this is a BUSY wait, and does not
- - guarentee FIFO order of waiters. In other news, Windows is a POS. -}
+ - guarantee FIFO order of waiters. In other news, Windows is a POS. -}
 waitToLock :: IO (Maybe lockhandle) -> IO lockhandle
 waitToLock locker = takelock
   where
