@@ -119,17 +119,22 @@ sendFile f (CodeObserver observer) ps = do
 			(findcode ph herr)
 		return (inout || inerr)
   where
-	p = wormHoleProcess (Param "send" : ps ++ [File f])
-	findcode ph h = findcode' =<< getwords ph h []
+	p = wormHoleProcess (ps ++ [Param "send", File f])
+	findcode ph h = hGetLineUntilExitOrEOF ph h >>= \case
+		Nothing -> return False
+		Just l -> ifM (findcode' (words l))
+			( drain ph h >> return True
+			, findcode ph h
+			)
 	findcode' [] = return False
 	findcode' (w:ws) = case mkCode w of
 		Just code -> do
 			_ <- tryPutMVar observer code
 			return True
 		Nothing -> findcode' ws
-	getwords ph h c = hGetLineUntilExitOrEOF ph h >>= \case
-		Nothing -> return $ concatMap words $ reverse c
-		Just l -> getwords ph h (l:c)
+	drain ph h = hGetLineUntilExitOrEOF ph h >>= \case
+		Just _ -> drain ph h
+		Nothing -> return ()
 
 -- | Receives a file. Once the receive is under way, the Code will be
 -- read from the CodeProducer, and fed to wormhole on stdin.
@@ -140,12 +145,12 @@ receiveFile f (CodeProducer producer) ps = runWormHoleProcess p $ \hin _hout _he
 	hFlush hin
 	return True
   where
-	p = wormHoleProcess $
+	p = wormHoleProcess $ ps ++
 		[ Param "receive"
 		, Param "--accept-file"
 		, Param "--output-file"
 		, File f
-		] ++ ps
+		]
 
 wormHoleProcess :: WormHoleParams -> CreateProcess
 wormHoleProcess = proc "wormhole" . toCommand
