@@ -1,11 +1,18 @@
 {- timestamp parsing and formatting
  -
- - Copyright 2015-2019 Joey Hess <id@joeyh.name>
+ - Copyright 2015-2023 Joey Hess <id@joeyh.name>
  -
  - License: BSD-2-clause
  -}
 
-module Utility.TimeStamp where
+{-# LANGUAGE CPP #-}
+
+module Utility.TimeStamp (
+	parserPOSIXTime,
+	parsePOSIXTime,
+	formatPOSIXTime,
+	truncateResolution,
+) where
 
 import Utility.Data
 
@@ -52,3 +59,14 @@ mkPOSIXTime n (d, dlen)
 
 formatPOSIXTime :: String -> POSIXTime -> String
 formatPOSIXTime fmt t = formatTime defaultTimeLocale fmt (posixSecondsToUTCTime t)
+
+{- Truncate the resolution to the specified number of decimal places. -}
+truncateResolution :: Int -> POSIXTime -> POSIXTime
+#if MIN_VERSION_time(1,9,1)
+truncateResolution n t = secondsToNominalDiffTime $
+	fromIntegral ((truncate (nominalDiffTimeToSeconds t * d)) :: Integer) / d
+  where
+	d = 10 ^ n
+#else
+truncateResolution _ t = t
+#endif
